from __future__ import annotations

from tcsoa.gen.BusinessObjects import Dataset, Item, ListOfValues, BusinessObject, Folder, WorkspaceObject, ItemRevision
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class FormAttributesInfo(TcBaseObj):
    """
    This structure contains the Form Attributes Information.
    
    :var formType: Form Type
    :var formPDs: A list of property descriptors
    """
    formType: str = ''
    formPDs: List[FormPropDescriptor] = ()


@dataclass
class FormInfo(TcBaseObj):
    """
    Holds the Information of Form to be created or updated
    
    :var clientId: Identifier that helps the client to track the object(s) created, required; should be unique for the
    input set
    :var formObject: The Form object, if null then create; otherwise, update.
    :var parentObject: Parent object of the Form to be created
    :var relationName: Relation name between Parent and Form to be created
    :var saveDB: If true, save the Form to database
    :var name: Name of the Form to be created
    :var description: Description of the Form to be created, can be an empty string.
    :var formType: Form Type of the Form to be created
    :var attributesMap: Form property map for the Form to be updated
    """
    clientId: str = ''
    formObject: BusinessObject = None
    parentObject: BusinessObject = None
    relationName: str = ''
    saveDB: bool = False
    name: str = ''
    description: str = ''
    formType: str = ''
    attributesMap: StringArrayMap = None


@dataclass
class FormPropDescriptor(TcBaseObj):
    """
    This structure contains information for Form Properties Descriptor.
    
    :var propName: Propety name
    :var displayName: Property display name
    :var attachedSpecifier: Attached specifier
    :var maxLength: Property maximumlLength
    :var interdependentProps: A list of interdependent properties
    :var propValueType: Property value type
    :var propType: Property type
    :var isDisplayable: true if property is displayable, otherwise false
    :var isArray: true if property is an array, otherwise false
    :var lov: List of values
    :var isRequired: true if propety is required, otherwise false
    :var isEnabled: true if property is enabled, otherwise false
    :var isModifiable: true if Property is Modifiable
    """
    propName: str = ''
    displayName: str = ''
    attachedSpecifier: int = 0
    maxLength: int = 0
    interdependentProps: List[str] = ()
    propValueType: int = 0
    propType: int = 0
    isDisplayable: bool = False
    isArray: bool = False
    lov: ListOfValues = None
    isRequired: bool = False
    isEnabled: bool = False
    isModifiable: bool = False


@dataclass
class GenerateUIDResponse(TcBaseObj):
    """
    Return structure for 'generateUID' operation
    
    :var uids: List of the UIDs that were generated
    :var serviceData: Contains any errors encountered during processing in the partial errors list.
    """
    uids: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class GetDatasetCreationRelatedInfoResponse(TcBaseObj):
    """
    Holds the Response data from getDatasetCreationRelatedInfo
    
    :var toolNames: List of Tool names
    :var newDatasetName: The name of the new Dataset, can be an empty string
    :var nameCanBeModified: If true, the name of the Dataset can be modified
    :var initValuePropertyRules: List of properties have the initialized values from property constant attachments
    :var serviceData: Standard 'ServiceData' member
    """
    toolNames: List[str] = ()
    newDatasetName: str = ''
    nameCanBeModified: bool = False
    initValuePropertyRules: List[str] = ()
    serviceData: ServiceData = None


@dataclass
class GetItemCreationRelatedInfoResponse(TcBaseObj):
    """
    This data structure contains a naming rules, property rules, form property descriptor, unit of measure, and
    ItemRevision type. Partial failure message will be returned in 'ServiceData'.
    
    :var complexValuePropertyRules: List of complex value property rules
    :var initValuePropertyRules: List of initial values
    :var patternMap: Pattern map (string/string)
    :var uoms: List of unit of measures
    :var formAttrs: List of Form attributes
    :var revTypeName: ItemRevision type name
    :var serviceData: The successful object ids, partial errors and failures
    """
    complexValuePropertyRules: List[str] = ()
    initValuePropertyRules: List[str] = ()
    patternMap: StringArrayMap = None
    uoms: List[str] = ()
    formAttrs: List[FormAttributesInfo] = ()
    revTypeName: str = ''
    serviceData: ServiceData = None


@dataclass
class GetItemFromIdInfo(TcBaseObj):
    """
    Input structure for getItemFromIdInfo, each of which contain an item id and optionally a list of revision ids which
    specify which Item Revisions to retrieve.
    
    :var itemId: Item id string for Item to retrieve, required
    :var revIds: Revision ids which specify which Item Revisions to retrieve, optional
    """
    itemId: str = ''
    revIds: List[str] = ()


@dataclass
class GetItemFromIdItemOutput(TcBaseObj):
    """
    This structure contains information for 'getItemFromId' operation.
    
    :var item: Item object reference of the item found
    :var itemRevOutput: List of 'GetItemFromIdItemRevOutput'
    """
    item: Item = None
    itemRevOutput: List[GetItemFromIdItemRevOutput] = ()


@dataclass
class GetItemFromIdItemRevOutput(TcBaseObj):
    """
    This structure contains an ItemRevision object and a list of found Dataset objects.
    
    :var itemRevision: ItemRevision object reference of the item revision found
    :var datasets: List of Dataset object references found
    """
    itemRevision: ItemRevision = None
    datasets: List[Dataset] = ()


@dataclass
class GetItemFromIdPref(TcBaseObj):
    """
    Input structure for 'getItemFromAttribute' used to filter the returned ItemRevision objects.
    
    :var prefs: A list of 'RelationFilter' structures for determining which ItemRevision objects to return
    """
    prefs: List[RelationFilter] = ()


@dataclass
class GetItemFromIdResponse(TcBaseObj):
    """
    Return structure for getItemFromId operation
    
    :var output: List of GetItemFromIdItemOutput
    :var serviceData: Standard ServiceData member
    """
    output: List[GetItemFromIdItemOutput] = ()
    serviceData: ServiceData = None


@dataclass
class MoveToNewFolderInfo(TcBaseObj):
    """
    Input structure for the 'moveToNewFolder' operation.
    
    
    :var oldFolder: Folder object reference of the old folder, whose content is going to move to new folder.
    :var newFolder: Folder object reference of the new folder, where object content is going to move, required. If not
    specified, processing will continue to the new input.
    :var objectsToMove: List of object references to be moved from the old folder to the new folder or copied to the
    new folder, required. If not specified, processing will continue to the next input.
    """
    oldFolder: Folder = None
    newFolder: Folder = None
    objectsToMove: List[BusinessObject] = ()


@dataclass
class MoveToNewFolderResponse(TcBaseObj):
    """
    Return structure for the 'moveToNewFolder' operation.
    
    
    :var serviceData: Contains the updated old folder, the new folder, and partial errors from this operation.
    """
    serviceData: ServiceData = None


@dataclass
class RelationFilter(TcBaseObj):
    """
    A filter for determining which ItemRevision objects to return.
    
    :var relationTypeName: The  relation type name that specifies the relation that relates the ItemRevision to the
    Dataset.
    :var objectTypeNames: A list of  Dataset object type names to return
    """
    relationTypeName: str = ''
    objectTypeNames: List[str] = ()


@dataclass
class SaveAsNewItemInfo(TcBaseObj):
    """
    Input structure for saveAsNewItem operation
    
    :var clientId: Used to uniquely identify the input (Required)
    :var itemRevision: Original Item Revision to be used for the SaveAsNewItem operation (Required)
    :var itemId: Item id string for creating new item, optional
    :var revId: Revision id string for creating new revision, optional
    :var name: Name string for creating new item/revision, optional
    :var description: Description string for creating new item/revision, optional
    :var folder: Folder object reference to attach Item to, if null, the Item will go to the default preference
    location for created objects.
    """
    clientId: str = ''
    itemRevision: ItemRevision = None
    itemId: str = ''
    revId: str = ''
    name: str = ''
    description: str = ''
    folder: Folder = None


@dataclass
class SaveAsNewItemOutput(TcBaseObj):
    """
    The data strucutre contains newly created object inforamtiobn.
    
    :var item: New Item object
    :var itemRev: New ItemRevision object
    """
    item: Item = None
    itemRev: ItemRevision = None


@dataclass
class SaveAsNewItemResponse(TcBaseObj):
    """
    The data structure contains return information for the operation.
    
    :var inputToNewItem: Map whose keys are the input clientIds and output values (newly created Item and ItemRevision
    objects) pairs ('string', 'SaveAsNewItemOutput')
    :var serviceData: Service data
    """
    inputToNewItem: InputToNewItemMap = None
    serviceData: ServiceData = None


@dataclass
class VecStruct(TcBaseObj):
    """
    This structure contains string list.
    
    :var stringVec: A list of strings
    """
    stringVec: List[str] = ()


@dataclass
class WhereReferencedInfo(TcBaseObj):
    """
    This data structure contains output information of referencer, relation and level.
    
    :var referencer: WorkspaceObject that references the input object
    :var relation: String name of the relation between the reference and the input object
    :var level: Level at which referencer was found.
    """
    referencer: WorkspaceObject = None
    relation: str = ''
    level: int = 0


@dataclass
class WhereReferencedOutput(TcBaseObj):
    """
    This data structure contains output information of reference, relation name and level for given input objects.
    
    :var inputObject: WorkspaceObject that is referenced by info
    :var info: Output information containing reference, relation name and level for 'inputObject'
    """
    inputObject: WorkspaceObject = None
    info: List[WhereReferencedInfo] = ()


@dataclass
class WhereReferencedResponse(TcBaseObj):
    """
    This data structure contains output information for a list of where referenced objects.
    
    :var output: List of information containing reference, relation name and level for input object
    :var serviceData: Service data
    """
    output: List[WhereReferencedOutput] = ()
    serviceData: ServiceData = None


@dataclass
class WhereUsedOutput(TcBaseObj):
    """
    This structure contains information for 'WhereUsedOutput'.
    
    :var inputObject: Input WorkspaceObject object reference for mapping to the output
    :var info: List of 'WhereUsedParentInfo' structures
    """
    inputObject: WorkspaceObject = None
    info: List[WhereUsedParentInfo] = ()


@dataclass
class WhereUsedParentInfo(TcBaseObj):
    """
    This structure contains Generic Parent Info.
    
    :var parentItemRev: Parent ItemRevision object reference which uses the given object
    :var level: The level at which the parent ItemRevision was found
    """
    parentItemRev: ItemRevision = None
    level: int = 0


@dataclass
class WhereUsedResponse(TcBaseObj):
    """
    'WhereUsedResponse' contains list of 'WhereUsedOutput'  structure. This structure contains list of Item and
    ItemRevision objects which are results of 'whereUsed' search.
    
    :var output: List of WhereUsedOutput structures
    :var serviceData: Standard 'ServiceData' member
    """
    output: List[WhereUsedOutput] = ()
    serviceData: ServiceData = None


@dataclass
class CreateFormsOutput(TcBaseObj):
    """
    This structure contains newly created form or updated form information.
    
    :var clientId: Identifier that helps the client to track the object(s) created or updated.
    :var form: The Form object created or updated
    """
    clientId: str = ''
    form: BusinessObject = None


@dataclass
class CreateOrUpdateFormsResponse(TcBaseObj):
    """
    Holds the Response of created or updated Forms
    
    :var outputs: List of created or updated Form objects
    :var serviceData: Standard 'ServiceData' member
    """
    outputs: List[CreateFormsOutput] = ()
    serviceData: ServiceData = None


"""
A map of 'InputToNewItem' '(string, SaveAsNewItemOutput').
"""
InputToNewItemMap = Dict[str, SaveAsNewItemOutput]


"""
A map of attribute names and desired value pairs (string/VecStruct).
"""
NameValueMap = Dict[str, VecStruct]


"""
Map of string array property names to values ('string, vector<string>').
"""
StringArrayMap = Dict[str, List[str]]



from __future__ import annotations

from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class BusinessObjectQueryClause(TcBaseObj):
    """
    Business object query clause
    
    :var propName: Property Name.
    :var propValue: Property Value.
    :var mathOperator: Math Operator.  Legal math operators can be =, !=, >, >=, <, <=, Contain, IS_NULL or IS_NOT_NULL
    according to the property type.
    :var logicOperator: Logic Operator.  Legal logic operators are "AND" and "OR".
    """
    propName: str = ''
    propValue: str = ''
    mathOperator: str = ''
    logicOperator: str = ''


@dataclass
class BusinessObjectQueryInput(TcBaseObj):
    """
    Business Object Query Input
    
    :var typeName: Name of business object type
    :var clauses: Query clauses in serach criteria.
    :var maxNumToReturn: Specified maximum number of objects to return.
    :var requestId: Unique ID used to register the query execution task. This can be used by the caller to cancel the
    time consuming query, the value can be generated by any unique string generator.
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure. This is currently not yet used by the return data elements, the caller can leave it empty.
    """
    typeName: str = ''
    clauses: List[BusinessObjectQueryClause] = ()
    maxNumToReturn: int = 0
    requestId: str = ''
    clientId: str = ''



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class MFGNodePropertyValues(TcBaseObj):
    """
    The structure that holds MFGNode along with property names and their values.
    
    :var node: The MFGNode in BOP structure.
    :var props: Map of Property Name Value Pair
    """
    node: BusinessObject = None
    props: MFGPropertyNameValueMap = None


@dataclass
class PropertyCollectorOutput(TcBaseObj):
    """
    Output structure containing MfgNode, set of Properties that are evaluated for the node and set of property values
    corresponding to the property names.
    
    :var contextType: Context Type of MfgNode
    :var nodeValueList: The vector of MFGNodePropertyValues.
    """
    contextType: str = ''
    nodeValueList: List[MFGNodePropertyValues] = ()


@dataclass
class PropertyCollectorVisitorInfo(TcBaseObj):
    """
    This structure contains list of properties to collect for the corresponding Traversal Rule Key
    
    :var traversalVisitorsKeys: The collection of type of MfgNode and its corresponding MfgContext type.
    :var propToCollectList: The list of properties to be evaluated for the type of MfgNode specified in
    traversalRuleKey.
    """
    traversalVisitorsKeys: List[TraversalKeyInfo] = ()
    propToCollectList: List[str] = ()


@dataclass
class TraversalKeyInfo(TcBaseObj):
    """
    This is a structure of MFGNode types and MFGContext
    
    :var types: set of MFGNode types
    :var context: context of MFGNodes
    """
    types: List[str] = ()
    context: str = ''


@dataclass
class TraversalRuleInfo(TcBaseObj):
    """
    This structure contains TraversalRuleKeys and their corresponding relations properties.
    
    :var traversalRuleKeys: The collection of type of MfgNode and its corresponding MfgContext type.
    :var targets: The list of properties to be used to traverse for current MfgNode to its sub nodes.
    """
    traversalRuleKeys: List[TraversalKeyInfo] = ()
    targets: List[str] = ()


@dataclass
class CollectPropertiesInputInfo(TcBaseObj):
    """
    This is a input structure containing rootNode, set of Traversal Rules and a set of Traversal Visitor info.
    
    :var clientId: Client ID
    :var rootNode: The root node to traverse from.
    :var traversalRuleList: The traversal rules to be used while traversing BOP structure.
    :var propVisitorList: The collection of type of MfgNodes and properties to collect for the type.
    """
    clientId: str = ''
    rootNode: BusinessObject = None
    traversalRuleList: List[TraversalRuleInfo] = ()
    propVisitorList: List[PropertyCollectorVisitorInfo] = ()


@dataclass
class CollectPropertiesOutput(TcBaseObj):
    """
    Structure containing client Id and PropertyCollectoroutput
    
    :var clientId: Client ID
    :var output: vector of PropertyCollectorOutput
    """
    clientId: str = ''
    output: List[PropertyCollectorOutput] = ()


@dataclass
class CollectPropertiesResponse(TcBaseObj):
    """
    Response structure containing service data
    
    :var outputList: The collection of MfgNode, property names and theit values.
    :var serviceData: service data containing partial error data
    """
    outputList: List[CollectPropertiesOutput] = ()
    serviceData: ServiceData = None


"""
This map holds the property name value pairs
"""
MFGPropertyNameValueMap = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, BOMLine
from enum import Enum
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class FormAttributeExpression(TcBaseObj):
    """
    FormAttributeExpression contains information to search
    on a single Form Attribute within a containing SearchExpressionInfo
    object.
    The search on the multiple values specified within one FormAttributeExpression are
    combined with a logical 'OR' operation.
    
    :var isItemForm: Item or ItemRevision form
    :var relationType: The Form relation type
    :var formType: Form type to be searched
    :var attributeName: The attribute of the above Form type
    to be searched
    :var attributeType: The type of the attribute to be searched on
    :var queryOperator: operator to use for search value comparison
    :var values: The values to test for
    """
    isItemForm: bool = False
    relationType: BusinessObject = None
    formType: BusinessObject = None
    attributeName: str = ''
    attributeType: AttributeType = None
    queryOperator: QueryOperator = None
    values: AttributeValues = None


@dataclass
class InClassExpression(TcBaseObj):
    """
    InClassExpression
    
    :var inClassClassNames: inClassClassNames
    :var inClassAttributeIDs: inClassAttributeIDs
    :var inClassAttributeValues: inClassAttributeValues
    """
    inClassClassNames: List[str] = ()
    inClassAttributeIDs: List[int] = ()
    inClassAttributeValues: List[str] = ()


@dataclass
class LogicalDesignatorAttribute(TcBaseObj):
    """
    an object that represents a single LD attribute.
    This type will be used in a vector that will represent a Logical Designator .
    
    :var name: the attribute name
    :var value: the attribute value
    """
    name: str = ''
    value: str = ''


@dataclass
class MFGSearchCriteria(TcBaseObj):
    """
    TBD
    
    :var objectTypes:  the types of objects to search
    :var logicalDesignator: logical designator data
    :var refinements: refinements on the search
    """
    objectTypes: List[BusinessObject] = ()
    logicalDesignator: List[LogicalDesignatorAttribute] = ()
    refinements: List[MFGSearchRefinementSet] = ()


@dataclass
class MFGSearchRefinement(TcBaseObj):
    """
    refinement on the search
    
    :var relatedScopes: a vector of additional scopes. These are not the scopes in which we expect to find the results.
    These are the scopes from which we start to impose the limitations.
    :var relations: a vector specifying the list of relations between the related scopes to the "searched" objects
    """
    relatedScopes: List[BusinessObject] = ()
    relations: List[RelationInfo] = ()


@dataclass
class MFGSearchRefinementSet(TcBaseObj):
    """
    TBD
    
    :var refinements: a vector of refinements. The refinements are combined with logical 'OR' between them
    """
    refinements: List[MFGSearchRefinement] = ()


@dataclass
class AttributeExpression(TcBaseObj):
    """
    AttributeExpression encapsulates information to build a query on
    the attribute of an Item or ItemRevision or their sub-classes in the structure search scope.
    * A set of these attribute expressions is contained
    inside a SearchExpressionInfo object.
    Search on multiple values specified within one AttributeExpression are
    combined with a logical 'OR' operation.
    
    :var className: Item or ItemRevision or their sub-classes
    :var attributeName: The attribute of the above 'className'
    to be searched on
    :var attributeType: The type of the attribute to be searched on
    :var queryOperator: operator to use for search value comparison
    :var values: values
    """
    className: str = ''
    attributeName: str = ''
    attributeType: AttributeType = None
    queryOperator: QueryOperator = None
    values: AttributeValues = None


@dataclass
class OccurrenceNoteExpression(TcBaseObj):
    """
    OccurrenceNoteExpression contains information to search
    on a single occurrence note type within a containing SearchExpressionInfo
    object.
    The search on the multiple values specified within one OccurrenceNoteExpression are
    combined with a logical 'OR' operation.
    
    :var noteType: Occurrence note type to search
    :var queryOperator: operator to use for search value comparison
    :var values: The list of values to search for
    """
    noteType: str = ''
    queryOperator: QueryOperator = None
    values: List[str] = ()


@dataclass
class AttributeValues(TcBaseObj):
    """
    AttributeValues data structure specifies data values to be bound to the various search expressions for
    execution of the search. It encapsulates the values to be specified for each of the data types that
    can occur in any of the attribute search expressions.
    
    :var boolValues: boolean values that any of the accompanying set of search expressions may use
    :var intValues: integer values that any of the accompanying set of search expressions may use
    :var doubleValues: double values that any of the accompanying set of search expressions may use
    :var stringValues: string values that any of the accompanying set of search expressions may use
    :var dateValues: date values that any of the accompanying set of search expressions may use
    :var tagValues: tag values that any of the accompanying set of search expressions may use
    """
    boolValues: List[bool] = ()
    intValues: List[int] = ()
    doubleValues: List[float] = ()
    stringValues: List[str] = ()
    dateValues: datetime = None
    tagValues: List[BusinessObject] = ()


@dataclass
class PlaneZone(TcBaseObj):
    """
    Planezone defines a single plane to search against.
    
    :var xvalue: X value of the direction unit vector of the plane
    :var yvalue: Y value of the direction unit vector of the plane
    :var zvalue: Z value of the direction unit vector of the plane
    :var displacement: Plane displacement in metres along the direction vector specified above
    """
    xvalue: float = 0.0
    yvalue: float = 0.0
    zvalue: float = 0.0
    displacement: float = 0.0


@dataclass
class PlaneZoneExpression(TcBaseObj):
    """
    PlaneZoneExpression represents a 3D plane to search against. It may define a search for all parts that intersect
    with the plane, or a search
    for all parts on one side of the plane.
    
    :var planzeZone: planzeZone
    :var planeZoneOperator: Operator to use for expression
    """
    planzeZone: PlaneZone = None
    planeZoneOperator: PlaneZoneOperator = None


@dataclass
class ProximityExpression(TcBaseObj):
    """
    ProximityExpression specifies one set of proximity criteria. The search
    results from each bomLine are combined together using an 'OR' operation.
    
    :var bomLines: BOM lines around which to search
    :var distance: Proximity distance in metres from the outer surface of the BomLine geometry
    :var includeChildBomLines: includeChildBomLines
    """
    bomLines: List[BOMLine] = ()
    distance: float = 0.0
    includeChildBomLines: bool = False


@dataclass
class RelationInfo(TcBaseObj):
    """
    information about the relation
    
    :var relationName: the name of the relation
    :var matchType: the match type
    :var objectClass: the class from which the relation starts (e.g. MfgOperation)
    """
    relationName: str = ''
    matchType: MatchType = None
    objectClass: str = ''


@dataclass
class SavedQueryExpression(TcBaseObj):
    """
    SavedQueryExpression defines a search expression encapsulated as a Teamcenter saved-query that returns an Item,
    ItemRevision
    or any of their sub-types. The saved query may be a pre-defined out of the box saved query or one created using
    the Teamcenter QueryBuilder application
    
    :var savedQuery: Tag of an existing saved query
    :var entries: Attribute entries that are to be searched for
    :var values: Values of the above entries to be searched for
    """
    savedQuery: BusinessObject = None
    entries: List[str] = ()
    values: List[str] = ()


@dataclass
class SearchBox(TcBaseObj):
    """
    SearchBox defines a single box zone to search in. It contains the definition of an axis-aligned box
    and an accompanying transform to describe it orientation.
    
    :var xmin: Plane defining the minimum extent along the x-axis
    :var ymin: Plane defining the minimum extent along the y-axis
    :var zmin: Plane defining the minimum extent along the z-axis
    :var xmax: Plane defining the maximum extent along the x-axis
    :var ymax: Plane defining the maximum extent along the y-axis
    :var zmax: Plane defining the maximum extent along the z-axis
    :var transform: Transform defining the orientation of the box ( empty vector denotes identity transform )
    """
    xmin: float = 0.0
    ymin: float = 0.0
    zmin: float = 0.0
    xmax: float = 0.0
    ymax: float = 0.0
    zmax: float = 0.0
    transform: List[float] = ()


@dataclass
class SearchBySizeExpression(TcBaseObj):
    """
    SearchBySizeExpression defines a search for parts with bounding boxes that are larger than or smaller than
    a specified size
    
    :var largerThan: Search for larger than if this is set to true, smaller than if set to false
    :var diagonalLength: Length of the largest bounding box diagonal in metres
    """
    largerThan: bool = False
    diagonalLength: bool = False


@dataclass
class SearchExpressionSet(TcBaseObj):
    """
    SearchExpression data structure specifies the full set of search expressions that are to be used to perform a
    single structure search. Each one of the expressions provided are combined together using a logical 'AND' between
    them.
    
    :var itemAndRevisionAttributeExpressions: itemAndRevisionAttributeExpressions
    :var occurrenceNoteExpressions: A collection of Occurrence Note attribute search expressions
    :var returnScopedSubTreesHit: Specifies whether the search result should return only the sub-set of scopeBomLines
    given as input as part of the SearchScope
    :var formAttributeExpressions: A collection of Form attribute search expressions
    :var proximitySearchExpressions: A collection of spatial proximity search expressions
    :var boxZoneExpressions: A collection of spatial box zone expressions
    :var planeZoneExpressions: A collection of spatial plane zone search expressions
    :var savedQueryExpressions: A collection of saved query search expressions
    :var inClassQueryExpressions: inClassQueryExpressions
    :var sizeSearchExpression: sizeSearchExpression
    :var doTrushapeRefinement: Specifies whether to perform trueshape refinement on the spatial search or not
    """
    itemAndRevisionAttributeExpressions: List[AttributeExpression] = ()
    occurrenceNoteExpressions: List[OccurrenceNoteExpression] = ()
    returnScopedSubTreesHit: bool = False
    formAttributeExpressions: List[FormAttributeExpression] = ()
    proximitySearchExpressions: List[ProximityExpression] = ()
    boxZoneExpressions: List[BoxZoneExpression] = ()
    planeZoneExpressions: List[PlaneZoneExpression] = ()
    savedQueryExpressions: List[SavedQueryExpression] = ()
    inClassQueryExpressions: List[InClassExpression] = ()
    sizeSearchExpression: SearchBySizeExpression = None
    doTrushapeRefinement: bool = False


@dataclass
class BoxZoneExpression(TcBaseObj):
    """
    BoxZoneExpression specifies one set of Box zone criteria. The search
    results from each SearchBox are combined together using an
    'OR' operation. The spatial search returns a match with the
    search expression if a object either lies inside or intersects the
    search boxes
    
    :var searchBoxes: Boxes around which to search
    :var boxOperator: Operator to use for expression
    """
    searchBoxes: List[SearchBox] = ()
    boxOperator: BoxOperator = None


@dataclass
class StructureSearchResultResponse(TcBaseObj):
    """
    StructureSearchResultResponse is the result set received from a single step in a structure search.
    
    :var serviceData: serviceData
    :var objects: A batch of Objects that satisfy the search criteria for this search
    :var objectDone: The number of objects in the structure that have been processed so far
    :var estimatedObjectsLeft: An estimate of the number of lines yet to be processed
    :var percentComplete: percentComplete
    :var searchCursor: Search cursor object that holds the current state of the search in the BOM session
    :var finished: A boolean value which when 'true' specifies the search is finished
    """
    serviceData: ServiceData = None
    objects: List[BusinessObject] = ()
    objectDone: int = 0
    estimatedObjectsLeft: int = 0
    percentComplete: float = 0.0
    searchCursor: BusinessObject = None
    finished: bool = False


class MatchType(Enum):
    """
    Example: Find all found welds points connected to parts in a given scope with a full match between the weld point
    connected parts and the selected parts list, i.e. all the parts the weld is connected to were selected by the user .
    """
    NotApplicableMatch = 'NotApplicableMatch'
    FullMatch = 'FullMatch'
    PartialMatch = 'PartialMatch'
    NoMatch = 'NoMatch'


class AttributeType(Enum):
    """
    AttributeType enumerated type defines the type of data passed in an attribute search expression
    """
    BooleanType = 'BooleanType'
    IntegerType = 'IntegerType'
    DoubleType = 'DoubleType'
    StringType = 'StringType'
    DateType = 'DateType'
    TagType = 'TagType'


class PlaneZoneOperator(Enum):
    """
    PlaneZoneOperator enumerated type defines the operator to use as part of a plane-zone search expression
    """
    PlaneAbove = 'PlaneAbove'
    PlaneBelow = 'PlaneBelow'
    PlaneIntersects = 'PlaneIntersects'
    PlaneAboveOrIntersects = 'PlaneAboveOrIntersects'
    PlaneBelowOrIntersects = 'PlaneBelowOrIntersects'


class QueryOperator(Enum):
    """
    QueryOperator  enumerated type defines the operator to be used for comparison in a attribute search expression
    """
    Equal = 'Equal'
    GreaterThan = 'GreaterThan'
    GreaterThanOrEqual = 'GreaterThanOrEqual'
    LessThan = 'LessThan'
    LessThanOrEqual = 'LessThanOrEqual'
    NotEqual = 'NotEqual'
    IsNull = 'IsNull'
    IsNotNull = 'IsNotNull'
    Like = 'Like'
    NotLike = 'NotLike'


class BoxOperator(Enum):
    """
    BoxOperator defines the operator to apply as part of a box-zone search expression.
    """
    Inside = 'Inside'
    Outside = 'Outside'
    Intersects = 'Intersects'
    InsideOrIntersects = 'InsideOrIntersects'
    OutsideOrIntersects = 'OutsideOrIntersects'



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMWindow
from typing import Dict, List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AllocationContextInput2(TcBaseObj):
    """
    The AllocationContextInput2 structure represents all the data necessary for creating an Allocation Context. The
    basic attributes that are required by ITK are passed as named elements in the structure. It will be used by the
    revised SOA createAllocationContext2() which also handles extended attributes.
    
    :var id: The ID of the AllocationMap object to be created. If empty, will be generated.
    :var name: The name of the AllocationMap object to be created. Optional input.
    :var type: The type of the AllocationMap object to be created. If type is not provided, the AllocationMap created
    will be of type AllocationMap.
    :var revision: The revision id for the AllocationRevisionMap object to be created.
    :var openedBOMWindows: List of BOMWindow business objects to be associated to the AllocationMap business object,
    where the created AllocationMap will be the context for the Allocations created between the BOMLines of these
    BOMWindows.
    :var attrValueMap: The map of  Attribute names and  value pairs to be used for AllocationMap business object
    creation of type string/string. The client calling this operation is responsible for converting the different
    property types (int , float, date etc) to string  using the appropriate to XXXString functions. Multi valued
    properties are represented with a comma separated string.
    """
    id: str = ''
    name: str = ''
    type: str = ''
    revision: str = ''
    openedBOMWindows: List[BOMWindow] = ()
    attrValueMap: AttrValueMap = None


"""
The AttrValueMap map represents Attribute Value pair map for the Allocation Map attributes.
"""
AttrValueMap = Dict[str, str]



from __future__ import annotations

from typing import List
from tcsoa.gen.ProjectManagementAw._2017_06.ScheduleManagementAw import MasterScheduleCreateInput
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ScheduleManagementAwService(TcService):

    @classmethod
    def createMasterSchedule(cls, createMasterInputs: List[MasterScheduleCreateInput], runInBackground: bool) -> ServiceData:
        """
        This operation creates new master Schedule and inserts one or more Schedule objects specified in the input,into
        a master Schedule. The information needed to create the master Schedule is specified in the
        MasterScheduleCreateInput structure. In case of background mode,this operation files an asynchronous request to
        create master Schedule and releases the client immediately so that the user can perform other operation.
        """
        return cls.execute_soa_method(
            method_name='createMasterSchedule',
            library='ProjectManagementAw',
            service_date='2017_06',
            service_name='ScheduleManagementAw',
            params={'createMasterInputs': createMasterInputs, 'runInBackground': runInBackground},
            response_cls=ServiceData,
        )

    @classmethod
    def createMasterScheduleAsync(cls, createMasterInputs: List[MasterScheduleCreateInput]) -> None:
        """
        This operation creates new master Schedule and inserts one or more Schedule objects specified in the input,
        into a master Schedule. The information needed to create the master Schedule is specified in the
        MasterScheduleCreateInput structure. This operation runs asynchronously in its own server in the background.
        """
        return cls.execute_soa_method(
            method_name='createMasterScheduleAsync',
            library='ProjectManagementAw',
            service_date='2017_06',
            service_name='ScheduleManagementAw',
            params={'createMasterInputs': createMasterInputs},
            response_cls=None,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ConfigurationContext, VisStructureContext
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateVisSCInfo(TcBaseObj):
    """
    Input structure used for creating VisStrucutreContext objects based on the given inputs.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify return data elements and partial
    errors associated with this input structure.
    
    :var configContext: A reference to an existing ConfigurationContext object that contains a portion of the
    configuration to be used by the output VisStructureContext object.
    
    :var topLine: Object reference of type BOMView or BOMViewRevision that is the topline of the configuration.
    
    :var occurrencesList: List of UID chain strings representing  the occurrences to be included in the structure
    recipe. If not supplied then the output VisStructureContext object will not have its occurrence_list property set.
    This will be interpreted as the top line of the BOMWindow was selected for this configuration recipe.
    
    :var staticStructureFile: IMANFile reference to the PLMXML static representation of the structure, (optional).
    """
    clientId: str = ''
    configContext: ConfigurationContext = None
    topLine: BusinessObject = None
    occurrencesList: List[str] = ()
    staticStructureFile: BusinessObject = None


@dataclass
class CreateVisSCOutput(TcBaseObj):
    """
    The output structure that contains the references to the created VisStructureContext object along with the
    corresponding 'clientId'.
    
    :var clientId: The unique string supplied by the caller used to match the output to the supplied input.
    :var structureRecipe: VisStructureContext object that records the configuration recipe based on the input
    configuration objects.
    """
    clientId: str = ''
    structureRecipe: VisStructureContext = None


@dataclass
class CreateVisSCResponse(TcBaseObj):
    """
    Response structure for the 'createVisSC' operation.
    
    :var output: List of 'CreateVisSCOutput' structures containing the VisStructureContext objects created based on the
    input configuration objects.
    
    :var serviceData: The Service Data.
    """
    output: List[CreateVisSCOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.gen.Rdv._2009_04.ContextManagement import GetGOPartSolutionsResponse, GetPastePrimeInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class ContextManagementService(TcService):

    @classmethod
    def getAllGOPartSolutions(cls, goBomLine: BOMLine) -> GetGOPartSolutionsResponse:
        """
        Get the required information for display of part solutions of selected GBE and its instantiating ABE
        """
        return cls.execute_soa_method(
            method_name='getAllGOPartSolutions',
            library='Rdv',
            service_date='2009_04',
            service_name='ContextManagement',
            params={'goBomLine': goBomLine},
            response_cls=GetGOPartSolutionsResponse,
        )

    @classmethod
    def getPastePrimeAttributes(cls, input: GetPastePrimeInfo) -> ServiceData:
        """
        This operation pastes source Architecture Breakdown Element (ABE) BOMLine to the target Architecture Breakdown
        (AB) BOMLine. It pastes all the parents (up to top level AB) of the source ABE under target AB. This operation
        also copies the variability, Named Variant Expressions (NVEs) and part solutions from the source BOMLine to
        target BOMLine.
        
        Use cases:
        This operation can be used to paste the source Architecture Breakdown Element BOMLine to target Architecture
        Breakdown BOMLine.
        
        Use case 1: Copy variability
        User needs to set flag value as 1 in GetPastePrimeInfo structure to copy only the variability from the source
        BOMLine to target BOMLine.
        
        Use case 2: Copy variability and NamedVariantExpressions
        User needs to set flag value as 2 in GetPastePrimeInfo structure to copy the variability and Named Variant
        Expressions from the source BOMLine to target BOMLine.
        
        Use case 3: Copy variability, NamedVariantExpressions and part solutions
        User needs to set flag value as 3 in GetPastePrimeInfo structure to copy the variability, Named Variant
        Expressions and part solutions from the source BOMLine to target BOMLine.
        """
        return cls.execute_soa_method(
            method_name='getPastePrimeAttributes',
            library='Rdv',
            service_date='2009_04',
            service_name='ContextManagement',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Configurator._2014_06.ConfiguratorManagement import ConfigExpression, ConfigExpressionTerm, BusinessObjectConfigExpression, Violation
from tcsoa.gen.BusinessObjects import Cfg0AbsFamily, Cfg0ConfiguratorPerspective
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class AvailableProductVariabilityOutput(TcBaseObj):
    """
    The response structure returned by getAvailableProductVariability.
    
    :var availabilityExpressions: A list of AvailableVariability expressions defining available variability. The number
    of elements in this list corresponds to the number of available values in the input families (parameter
    familiesToTest). If an empty familiesToTest list was specified this vector would be empty.
    :var voilations: A list of constraints that were violated for a given user selected values/input expression.
    :var suggestedSatisfyingExpr: A sample expression satisfying the given input expression and existing set of
    constraint in the configurator context. This is an output expression containing system suggested features from the
    available features.
    :var serviceData: The service data for errors and returned objects.
    """
    availabilityExpressions: List[AvailableVariability] = ()
    voilations: List[Violation] = ()
    suggestedSatisfyingExpr: ConfigExpression = None
    serviceData: ServiceData = None


@dataclass
class AvailableVariability(TcBaseObj):
    """
    Defines available features for a given family along with the default value for operation
    getAvailableProductVariability. The AvailableVariability structure is interpreted as "NULL" if it has the following
    values set for its parameters: 
    defaultValue: NULL structure
    availability: empty list
    
    :var defaultValue: Specifies the configuration expression term default value. If no default rules exist, or they
    select a default value such that it is ruled out due to other constraints, this will be an empty
    ConfigExpressionTerm object to indicate a non-existing default value.
    :var availability: The list of configuration expressions referencing values only from the requested families. This
    is a list of discrete values or ranges.
    """
    defaultValue: ConfigExpressionTerm = None
    availability: List[ConfigExpressionTerm] = ()


@dataclass
class AvailableProductVariabilityInput(TcBaseObj):
    """
    It is the object containing input information required to compute the available product variability.
    
    :var criteriaExpression: The input criteria expression for which the available product variability is to be
    calculated.
    :var familiesToTest: The list of configuration families of which available values are requested. 
    :var context: Configurator context details. 
    :var applyConstraints: How to apply the constraints. The action is a bitwise OR the following values: 
    0: disable configurator constraint evaluation 
    1: enable configurator constraint evaluation 
    2: skip constraints if they only reference unset families 
    4: skip constraints that don't reference all configExpression families 
    8: report warnings in addition to errors 
    16: report informational messages 
    32: don't stop if a violation is encountered (use with care) 
    64: treat constraints with warning severity as if they had error severity.
    
    1024: skip criteria update based on validation rules (For example configurator exclusion rule). If this flag is not
    set then the operation will update the variant criteria after applying validation rules.
    
    2048: enable Availability Constraint evaluation. 
    """
    criteriaExpression: BusinessObjectConfigExpression = None
    familiesToTest: List[Cfg0AbsFamily] = ()
    context: Cfg0ConfiguratorPerspective = None
    applyConstraints: int = 0



from __future__ import annotations

from tcsoa.gen.Internal.AWS2._2019_06.AdvancedSavedSearch import Awp0UpdateAdvancedSavedSearchInput, Awp0AdvancedSavedSearchResponse, Awp0CreateAdvancedSavedSearchInput
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.AWS2._2016_03.Finder import ColumnConfigInput
from tcsoa.gen.Internal.AWS2._2019_06.RequirementsManagement import FullTextResponse, RequirementInput
from typing import List
from tcsoa.gen.Internal.AWS2._2019_06.Finder import SearchResponse6, SaveColumnConfigData2, SearchInput3
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class DataManagementService(TcService):

    @classmethod
    def modifyFavorites(cls, favorites: List[BusinessObject], action: str) -> ServiceData:
        """
        This operation updates the user's favorites by adding or removing the objects specified with the input.
        
        Favorites are stored per user in the database as Teamcenter preference: "MyFavorites"
        
        The operation takes list of objects and the action as input whether to add or remove and finalizes the complete
        list of favorites before they are saved to the database. This implies that the caller of the operation needs to
        send delta list of objects to add or remove to the user's favorites.
        """
        return cls.execute_soa_method(
            method_name='modifyFavorites',
            library='Internal-AWS2',
            service_date='2019_06',
            service_name='DataManagement',
            params={'favorites': favorites, 'action': action},
            response_cls=ServiceData,
        )


class FinderService(TcService):

    @classmethod
    def performSearchViewModel4(cls, searchInput: SearchInput3, columnConfigInput: ColumnConfigInput, saveColumnConfigData: SaveColumnConfigData2, inflateProperties: bool, noServiceData: bool) -> SearchResponse6:
        """
        This operation routes a search request to a specific provider specified as providerName in the searchInput
        structures.
        
        The framework allows a custom solution to be able to provide a new specific provider to collect data, perform
        sorting and filtering. The provider can be a User Inbox retriever or Full Text searcher, for example. The new
        data provider can be encapsulated via a new runtime business object from Fnd0BaseProvider class. The
        implementation is done using its fnd0performSearch operation.
        
        RuntimeBusinessObject 
        ---- Fnd0BaseProvider (foundation template) 
        -------- Fnd0GetChildrenProvider(foundation template) 
        -------- Awp0FullTextSearchProvider (aws template)
        -------- Awp0TaskSearchProvider (aws template) 
        -------- Aos0TestProvider (aosinternal template) 
        -------- etc.
        
        This operation provides a common interface to send the request to and receive the response from a new data
        provider. Ultimately it allows common framework in UI to support filter, pagination, and sorting. 
        
        This operation allows the caller to send the search input, filter values, and sorting data. These input values
        are then to be passed to the fnd0performSearch operation on the server to collect, sort, and filter its
        results. 
        
        The first two input parameters are important. The first input is the provider name. This is a string to
        represent the type name of RuntimeBusinessObject which this request should be routed to. If the template that
        contains the class is not installed, a partial error 217016 is returned. The second input is the search input
        map. The key is different per each provider. For example, for Full Text searcher, the input key would be a
        searchString. For User Inbox retriever, the input key would be an Inbox Type. The fnd0performSearch
        implementation for each provider takes into account the key name as it is used to store the values in
        OperationInput object. 
        
        The internal property name is passed as part of search input to allow grouping of business objects. The
        grouping allows for added functionality such as color coding, etc. If no internal property name is passed,
        default value is assumed as "object_type" to group the business objects. 
        
        If new list of column config data is passed in to be saved, it will get saved in the database for the login
        user scope. This will be done prior to performing the search. 
        
        The column configuration input has the criteria needed to query for the columns configured for the provided
        client and client scope URI. The final list of columns are identified based on the types returned back in the
        search results and the operation type (Union / Intersection / Configured ) configured for the client scope URI.
        Finally the determined column attributes and the attributes to be inflated passed in as part of search input
        criteria are combined. This combined list of attributes along with the business objects found during search are
        added to the Service data.
        
        Use cases:
        Use Case 1: User clicks on Inbox tile
        A search request will be made by passing in the needed criteria. In this case the provider name is
        "Awp0InboxProvider". It queries for the data and finds the effective columns for the client scope URI passed in
        through column config input. The results are finally shown in the view type selected in client. If table view
        is selected the returned effective columns will be shown.
        
        Use Case 2: User performs search by entering a keyword in global search input box. 
        A search request will be made by passing in the needed criteria that includes the keyword user has typed in. In
        this case the provider name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective
        columns for the client scope URI passed in through column config input. The results are finally shown in the
        view type selected in client. If table view is selected the returned effective columns will be shown. 
        
        Use Case 3: User tries to save new column configurations after performing search. 
        The new list of column configurations will be saved for the login user scope. Later the search request will be
        made by passing in the needed criteria that includes the keyword user had typed in. In this case the provider
        name is "Awp0FullTextSearchProvider". It queries for the data and finds the effective columns for the client
        scope URI passed in through column config input. The results are finally shown in the view type selected in
        client. If table view is selected the returned effective columns will be shown.
        
        ---
        """
        return cls.execute_soa_method(
            method_name='performSearchViewModel4',
            library='Internal-AWS2',
            service_date='2019_06',
            service_name='Finder',
            params={'searchInput': searchInput, 'columnConfigInput': columnConfigInput, 'saveColumnConfigData': saveColumnConfigData, 'inflateProperties': inflateProperties, 'noServiceData': noServiceData},
            response_cls=SearchResponse6,
        )


class AdvancedSavedSearchService(TcService):

    @classmethod
    def updateAdvancedSavedSearch(cls, inputs: List[Awp0UpdateAdvancedSavedSearchInput]) -> Awp0AdvancedSavedSearchResponse:
        """
        This operation updates advanced saved searches for given inputs.
        """
        return cls.execute_soa_method(
            method_name='updateAdvancedSavedSearch',
            library='Internal-AWS2',
            service_date='2019_06',
            service_name='AdvancedSavedSearch',
            params={'inputs': inputs},
            response_cls=Awp0AdvancedSavedSearchResponse,
        )

    @classmethod
    def createAdvancedSavedSearch(cls, inputs: List[Awp0CreateAdvancedSavedSearchInput]) -> Awp0AdvancedSavedSearchResponse:
        """
        This operation creates advanced saved searches for given inputs.
        """
        return cls.execute_soa_method(
            method_name='createAdvancedSavedSearch',
            library='Internal-AWS2',
            service_date='2019_06',
            service_name='AdvancedSavedSearch',
            params={'inputs': inputs},
            response_cls=Awp0AdvancedSavedSearchResponse,
        )


class RequirementsManagementService(TcService):

    @classmethod
    def getFullTextVersionInfo(cls, inputs: List[RequirementInput]) -> FullTextResponse:
        """
        This operation retrieves the information about the FullText dataset versions for the given Requirements. It
        also retrieves the contents and properties of the FullText dataset which are displayed in the user interface.
        The following partial errors may be returned:
        159001: An internal error has occurred in the Requirements Management module. Please report this error to your
        system administrator.
        141308: The type of operation to perform in invalid.
        141309: The selected input object does not contain FullText object.
        
        Use cases:
        Use Case 1: Get FullText dataset versions and contents
        User selects a Requirement in Home folder. User goes to History tab where document history list will be
        displayed.This document history list will contain information about all the Requirement Revision and its
        FullText dataset versions and its contents. User selects any two FullText dataset versions and clicks on
        compare which will give compare HTML report.
        
        Use Case 2: Get FullText dataset versions during Freeze of derived Requirement
        User loads the requirement specification in ACE (Active Content Experience) panel. User clicks on 'Reuse'
        command which shows the 'Reuse and Derived' panel. User selects 'Create a derived Specification' and selects a
        tracelink type and clicks on Create button. This creates a derived of the existing Requirement specification.
        User selects a derived Requirement and clicks on 'Freeze' option which shows the FullText dataset versions.
        User will pick a version and clicks on Ok button. This will freeze the Requirement and it will show the
        contents from the dataset version that user had picked.
        """
        return cls.execute_soa_method(
            method_name='getFullTextVersionInfo',
            library='Internal-AWS2',
            service_date='2019_06',
            service_name='RequirementsManagement',
            params={'inputs': inputs},
            response_cls=FullTextResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class VariantConfigurationCriteria(TcBaseObj):
    """
    Formula or VariantRule that needs to be applied on the window.
    
    :var variantConfiguratorFormula: variant criteria derived from variant formula or from variant rule created using
    Product Configurator application or from user selections in Variant Configuration View, that needs to be applied on
    the window.
    :var savedVariantRules: List of saved variant rules created via Product Configurator application that needs to be
    applied on the window. In case of multiple saved variant rule,  they are ORed and then applied to the window. Only
    the VariantRule expressions are considered for configuration.
    """
    variantConfiguratorFormula: str = ''
    savedVariantRules: List[BusinessObject] = ()



from __future__ import annotations

from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class VendorRoleData(TcBaseObj):
    """
    This is the structure containing VendorRole information.
    
    :var clientId: A unique string supplied by the caller. This ID is used to identify the Partial Errors associated
    with this 'VendorRoleData' input.
    :var description: Description of the object to be created.
    :var roleType: VendorRole type, it shall be Manufacturer,Supplier or Distributor
    :var remove: Flag to indicate VendorRole is to be added or removed.
    """
    clientId: str = ''
    description: str = ''
    roleType: str = ''
    remove: bool = False



from tcsoa.gen.Query._2013_05.services import SavedQueryService as imp0
from tcsoa.gen.Query._2007_01.services import SavedQueryService as imp1
from tcsoa.gen.Query._2006_03.services import SavedQueryService as imp2
from tcsoa.gen.Query._2018_11.services import SavedQueryService as imp3
from tcsoa.gen.Query._2019_06.services import SavedQueryService as imp4
from tcsoa.gen.Query._2020_04.services import SavedQueryService as imp5
from tcsoa.gen.Query._2010_09.services import SavedQueryService as imp6
from tcsoa.gen.Query._2007_06.services import SavedQueryService as imp7
from tcsoa.gen.Query._2007_09.services import SavedQueryService as imp8
from tcsoa.gen.Query._2008_06.services import SavedQueryService as imp9
from tcsoa.gen.Query._2010_04.services import SavedQueryService as imp10
from tcsoa.gen.Query._2007_06.services import FinderService as imp11
from tcsoa.gen.Query._2014_11.services import FinderService as imp12
from tcsoa.gen.Query._2012_10.services import FinderService as imp13
from tcsoa.utils import TcService


class SavedQueryService(TcService):
    createSavedQueries = imp0.createSavedQueries
    deleteQueryCriterias = imp1.deleteQueryCriterias
    describeSavedQueries = imp2.describeSavedQueries
    executeBOQueriesWithContext = imp3.executeBOQueriesWithContext
    executeBOQueriesWithSort = imp4.executeBOQueriesWithSort
    executeBOQueriesWithSortAndContext = imp5.executeBOQueriesWithSortAndContext
    executeBusinessObjectQueries = imp6.executeBusinessObjectQueries
    executeSavedQueries = imp7.executeSavedQueries
    executeSavedQueries2 = imp8.executeSavedQueries
    executeSavedQueries3 = imp9.executeSavedQueries
    executeSavedQuery = imp2.executeSavedQuery
    findSavedQueries = imp10.findSavedQueries
    getSavedQueries = imp2.getSavedQueries
    reorderSavedQueryCriterias = imp1.reorderSavedQueryCriterias
    retrieveQueryCriterias = imp1.retrieveQueryCriterias
    retrieveSearchCriteria = imp7.retrieveSearchCriteria
    saveQueryCriterias = imp1.saveQueryCriterias
    saveSearchCriteria = imp7.saveSearchCriteria


class FinderService(TcService):
    findWorkspaceObjects = imp11.findWorkspaceObjects
    groupObjectsByProperties = imp12.groupObjectsByProperties
    performSearch = imp13.performSearch
    performSearch2 = imp12.performSearch



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, NamedVariantExpression
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetVariantExprAllDataResponse(TcBaseObj):
    """
    This structure contains the XO chart related information and the 'ServiceData' object.
    
    :var nveData: An object containing all the NVEs information and their XO chart data related information.
    :var serviceData: An object of ServiceData which contains any exceptions if occurred during creation of XO chart
    data.
    """
    nveData: VariantExprAllData = None
    serviceData: ServiceData = None


@dataclass
class VariantExprAllData(TcBaseObj):
    """
    This structure contains all the NVEs information and their XO chart data related information.
    
    :var varExprXOData: An object containing information required to display the XO chart
    :var codeDescData: List of VariantExprCodeDescData structure
    """
    varExprXOData: VariantExprXOChartData = None
    codeDescData: List[VariantExprCodeDescData] = ()


@dataclass
class VariantExprCodeDescData(TcBaseObj):
    """
    This structure is used to store the NVE related information like NVE object reference, name and description.
    
    :var nve: The NVE object
    :var code: Name of the Named Variant Expression
    :var desc: Description of the NamedVariantExpression
    """
    nve: NamedVariantExpression = None
    code: str = ''
    desc: str = ''


@dataclass
class VariantExprXOChartData(TcBaseObj):
    """
    This structure contains the XO chart information such as table header strings, number of rows and columns.
    
    :var noOfColHeaders: Number of column header to display in the UI
    :var colHeaderExprs: list of col header expressions
    :var noOfRows: Number of rows to display in the UI
    :var noOfCols: Number of columns in the UI
    :var noOfTableChars: Number of table characters
    :var tableChars: The octal string representation of the table char bytes
    :var colHeaderExprStrs: Values for the column headers. Format of value will be like option name=value ex. Color=Blue
    """
    noOfColHeaders: int = 0
    colHeaderExprs: List[BusinessObject] = ()
    noOfRows: int = 0
    noOfCols: int = 0
    noOfTableChars: int = 0
    tableChars: str = ''
    colHeaderExprStrs: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ListOfValues
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class LOVAttachment(TcBaseObj):
    """
    Structure to hold property and its LOV attachment.
    
    :var propName: Name of the property to  which LOV attachment return.
    :var lov: Attached LOV object.
    """
    propName: str = ''
    lov: ListOfValues = None


@dataclass
class LOVAttachmentsInput(TcBaseObj):
    """
    A structure holding objects and common properties for which LOV attachments are to be returned.
    
    :var objects: Instances of any business object for which LOV attachments are being evaluated.
    :var properties: Names of common properties for which LOV attachments to be returned for each instance in 'objects'
    """
    objects: List[BusinessObject] = ()
    properties: List[str] = ()


@dataclass
class LOVAttachmentsResponse(TcBaseObj):
    """
    LOV attachments for the given object properties.
    
    :var lovAttachments: A list of objects and its properties and associated LOV attachments.
    :var serviceData: The Service Data.
    """
    lovAttachments: InpoutObjectToLOVAttachmentsMap = None
    serviceData: ServiceData = None


"""
A map of input objects to list of LOVattachments structure.
"""
InpoutObjectToLOVAttachmentsMap = Dict[BusinessObject, List[LOVAttachment]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class BOMLinePair(TcBaseObj):
    """
    This structure provides a pair of BOM lines.
    
    :var source: the source BOM line
    :var target: the target BOM line
    """
    source: BOMLine = None
    target: BOMLine = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class TcRAReportsCriteria(TcBaseObj):
    """
    Criteria needed to retrieve the URL for the specified TcRA operation.
    
    :var messageName: It designates desired TcRA report definition operation, e.g. retrieve, view, edit, delete or set
    permission.
    :var reportDefinition: The report definition ID.
    :var contextObjects: A list of ID's representing context object(s) (required for item reports).
    """
    messageName: str = ''
    reportDefinition: BusinessObject = None
    contextObjects: List[BusinessObject] = ()


@dataclass
class ConstructReportURLResponse(TcBaseObj):
    """
    The response object from constructReportURL operation.
    
    :var url: The URL string for the TcRA servlet.
    :var serviceData: The ServiceData which contains the error stack.
    """
    url: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2016_03.OccurrenceManagement import SubsetInput2, AddToBookmarkResp2, ReplaceElementResponse, EffectivityInput, ReplaceInput, AddToBookmarkInputData2
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Internal.ActiveWorkspaceBom._2015_07.OccurrenceManagement import SubsetResponse
from tcsoa.utils import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def addOrRemoveOccurrenceEffectivities(cls, input: EffectivityInput) -> ServiceData:
        """
        This operation adds or removes Effectivity objects to/from the input list of Awb0Element(s). The impacted
        Awb0Element(s) are reconfigured with new effectivities.
        """
        return cls.execute_soa_method(
            method_name='addOrRemoveOccurrenceEffectivities',
            library='Internal-ActiveWorkspaceBom',
            service_date='2016_03',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def addToBookmark2(cls, input: AddToBookmarkInputData2) -> AddToBookmarkResp2:
        """
        This operation adds a product with current configuration parameters to the Awb0AutoBookmark associated with the
        input Awb0SavedBookmark. This operation does not update the Awb0SavedBookmark itself.
        """
        return cls.execute_soa_method(
            method_name='addToBookmark2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2016_03',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=AddToBookmarkResp2,
        )

    @classmethod
    def getSubsetInfo2(cls, subsetInputs: List[SubsetInput2]) -> SubsetResponse:
        """
        This operation retrieves filters and recipes which are used to find matching Awb0Element objects for input
        Awb0ProductContextInfo.
        """
        return cls.execute_soa_method(
            method_name='getSubsetInfo2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2016_03',
            service_name='OccurrenceManagement',
            params={'subsetInputs': subsetInputs},
            response_cls=SubsetResponse,
        )

    @classmethod
    def replaceElement(cls, input: ReplaceInput) -> ReplaceElementResponse:
        """
        This operation replaces an underlying object of the Awb0Element with input BusinessObject. The
        Awb0ProductContextInfo provides configuration information for the 'replacement'.
        """
        return cls.execute_soa_method(
            method_name='replaceElement',
            library='Internal-ActiveWorkspaceBom',
            service_date='2016_03',
            service_name='OccurrenceManagement',
            params={'input': input},
            response_cls=ReplaceElementResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import Awb0Element, Item, Awb0ProductContextInfo, BusinessObject, VariantRule, RevisionRule
from enum import Enum
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class OccurrenceFilter(TcBaseObj):
    """
    'OccurrenceFilter' specifies the search filtering and sorting criteria if filtering and sorting of the resulting
    occurrences is desired.
    
    :var searchInput: Full Text Filter information
    """
    searchInput: SearchInput = None


@dataclass
class OccurrenceInfo(TcBaseObj):
    """
    'OccurrenceInfo' is the information about one occurrence that is returned as part of the result. It contains an
    occurrence that is part of the configured structure being queried and that passes the filter criteria, as well as
    the attachments requested.
    
    :var occurrence: Instance of the Awb0Element.
    :var attachedObjects: A list of attached objects if attachments were requested as part of the
    'AttachedDataCriteria' in the input. This list will be empty if no attachments were requested.
    """
    occurrence: Awb0Element = None
    attachedObjects: List[AttachedObjectInfo] = ()


@dataclass
class AttachedDataCriteria(TcBaseObj):
    """
    'AttachedDataCriteria' specifies which atachments of occurrences or the attachments of the underlying persisted
    occurrence model, to return with each resulting occurrence. The TC_relation objects specified here are traversed to
    retrieved attached objects for each occurrence.
    
    :var relationName: The relation to traverse from the Archetype for fetching attached data
    :var attachedObjectType: The type of the attached object to traverse to using the above relation
    :var namedReferences: Specific Named Reference to return in the attached object. Applies only if the attached
    object is an Dataset. If not named references are specified, all named references are returned.
    :var returnAttachedObject: If true, the attached object is returned in output.
    :var returnNamedRefObject: If true, the named reference object is returned as part of output.
    :var returnFileTickets: If true, the file tickets for the given named ref object is returned in the output.
    """
    relationName: str = ''
    attachedObjectType: str = ''
    namedReferences: List[str] = ()
    returnAttachedObject: bool = False
    returnNamedRefObject: bool = False
    returnFileTickets: bool = False


@dataclass
class AttachedObjectInfo(TcBaseObj):
    """
    'AttachedObjectInfo' returns attached objects as requested by the 'AttachedDataCriteria'.
    
    :var relationName: Relation for the attached object
    :var attachedObject: Attached Object. Null if this was not requested in the criteria
    :var namedReferenceName: Named Reference Name. This will be empty if the attached object is not an dataset or if
    named references were not requested.
    :var namedRefObject: Named Reference Object. This will be null if the attached object is not an dataset or if the
    named reference object was not requested.
    :var fileTicket: File Ticket for the Named Reference Object. This will be empty if the attached object is not an
    dataset or if the named reference object is not an ImanFile.
    """
    relationName: str = ''
    attachedObject: BusinessObject = None
    namedReferenceName: str = ''
    namedRefObject: BusinessObject = None
    fileTicket: str = ''


@dataclass
class ProductInfoData(TcBaseObj):
    """
    This structure contains the Awb0ProductContextInfo and the root Awb0Element for the product.
    
    :var productCtxInfo: The configuration of the product saved in the Awb0SavedBookmark or Awb0AutoBookmark.
    :var rootElement: The root Awb0Element of the product.
    """
    productCtxInfo: Awb0ProductContextInfo = None
    rootElement: Awb0Element = None


@dataclass
class ProductOccurrencesInput(TcBaseObj):
    """
    The 'GetOccurrencesInProduct' supplies the product and the configuration rules to use to configure the product and
    get back a page of occurrences. Optionally, filters and sorting criteria may be provided to filter or sort the
    resulting occurrences. You may also optionally supply criteria to bring back attachments that may be attached to
    occurrences with a TC_relation.
    
    :var product: The product which needs to be configured. Could be an ItemRev, CollaborativeDesign or a
    StructureContextObject.
    
     If Structure Context is passed in, the configuration below need not be specified ( the configuration information
    is ignored if this is specified ).
    :var inputCtxt: Common input context including filter, configuration, attached data criteria and page size.
    :var firstLevelOnly: If true, only the first level instances are returned.
    """
    product: BusinessObject = None
    inputCtxt: InputContext = None
    firstLevelOnly: bool = False


@dataclass
class SearchFilter(TcBaseObj):
    """
    'SearchFilter' specifies the details of a filter. It indicates if it is a string type, date type or numeric type of
    search filter that is to be applied to the occurrences of the requested product.
    
    :var searchFilterType: The type of search filter. Valid values are "StringFilter", "DateFilter", "NumericFilter".
    :var stringValue: The value for a string filter. This field is applicable only if the "searchFilterType" field is
    set to "StringFilter".
    :var startDateValue: The starting value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var endDateValue: The ending value for a date filter. This field is applicable only if the "searchFilterType"
    field is set to "DateFilter".
    :var startNumericValue: The starting value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var endNumericValue: The ending value for a numeric filter. This field is applicable only if the
    "searchFilterType" field is set to "NumericFilter".
    :var count: The number of values in the filter. This field is populated on the service response and is ignored on
    the service input.
    :var selected: A flag that indicates if the filter was previously selected and used to filter the search results.
    This field is populated on the service response and is ignored on the service input.
    :var startEndRange: The 'gap' used to generate the start and end values
    """
    searchFilterType: SearchFilterType = None
    stringValue: str = ''
    startDateValue: datetime = None
    endDateValue: datetime = None
    startNumericValue: float = 0.0
    endNumericValue: float = 0.0
    count: int = 0
    selected: bool = False
    startEndRange: str = ''


@dataclass
class SearchFilterField(TcBaseObj):
    """
    'SearchFilterField' returns the resulting configured products found, that contain the queried object. This data
    structure also returns a cursor which can be sent back to a call to in order to rerieve more pages of results.
    
    :var internalName: The internal name for the search filter field.
    :var displayName: The display name for the search filter field.
    :var defaultFilterValueDisplayCount: The default number of search filter values to display within the search filter
    field.
    """
    internalName: str = ''
    displayName: str = ''
    defaultFilterValueDisplayCount: int = 0


@dataclass
class SearchInput(TcBaseObj):
    """
    'SearchInput' contains input filtering criteria that will be used to filter and sort the occurrences in the
    requested product.
    
    :var searchString: The string to search for.
    :var searchFilterMap: The map containing the list of search filters for each search filter field.
    :var searchSortCriteria: The criteria to use to sort the results.
    :var searchFilterFieldSortType: The sorting type to use to order the search filter categories in the response. The
    acceptable values are: "Alphabetical", "Priority".
    """
    searchString: str = ''
    searchFilterMap: SearchFilterMap = None
    searchSortCriteria: List[SearchSortCriteria] = ()
    searchFilterFieldSortType: SearchFilterFieldSortType = None


@dataclass
class SearchSortCriteria(TcBaseObj):
    """
    'SearchSortCriteria' specifies the criteria to use to sort the results that are retrieved. It provides the field to
    sort and the direction to sort it in.
    
    :var fieldName: The name of the field on which to perform the sorting.
    :var sortDirection: The direction in which the sorting needs to be performed - ascending or descending.
    """
    fieldName: str = ''
    sortDirection: SortDirection = None


@dataclass
class ChildOccsLoadNxtCursor(TcBaseObj):
    """
    The cusor has the information about loading the next set of child occurrences for given parents. It also has
    Awb0ProductContextInfo and input context which captures the product and configuration information.
    
    :var contextInfo: The product context associated with the occurrence.
    :var parentOccurrences: Parent occurrences for which children were desired
    :var inputCtxt: Common input context including filter, configuration, attached data criteria, page size and
    structure context object. If structure context object info given then configuration information is not required.
    :var firstLevelOnly: Return first level instances only
    :var startIndex: Cursor start position for the result occurrences returned so far
    :var endIndex: Cursor end position for the result occurrences returned so far
    :var pageSize: This indicate the number of occurrences needs to be returned.
    :var cursorData: DEPRECATED: Cursor data
    """
    contextInfo: Awb0ProductContextInfo = None
    parentOccurrences: List[Awb0Element] = ()
    inputCtxt: InputContext = None
    firstLevelOnly: bool = False
    startIndex: int = 0
    endIndex: int = 0
    pageSize: int = 0
    cursorData: List[str] = ()


@dataclass
class ChildOccurrenceInfo(TcBaseObj):
    """
    The list of child occurrences for a given parent. It also has the information about the paging position of the
    given list of occurrences. The paging position has values 'startReached', 'endReached' to identify the position and
    supports two way scrolling. Optionally the 'focusOccurrence' used to search a particular object of Awb0Element.
    
    :var contextInfo: The product context info associated with the occurrence.
    :var parent: Parent occurrence for which children were desired.
    :var startReached: Reached the first page of occurrences
    :var endReached: Reached the last page of occurrences
    :var childOccurrences: One page of child occurrences.
    :var focusOccurrence: The Awb0Element objects for given focussedOccurrence or for gievn focussedOccId.
    """
    contextInfo: Awb0ProductContextInfo = None
    parent: Awb0Element = None
    startReached: bool = False
    endReached: bool = False
    childOccurrences: List[OccurrenceInfo] = ()
    focusOccurrence: Awb0Element = None


@dataclass
class ChildOccurrenceInfoList(TcBaseObj):
    """
    The list of child Awb0Element along with Awb0ProductContextInfo containing top level product configuration
    information along with next level cursor.
    
    :var contextInfo: The product context info associated with the occurence.
    :var childOccInfos: Vector of child occurrence information for parents in a product context.
    :var cursor: Cursor that represents point to which results have been retrieved
    """
    contextInfo: Awb0ProductContextInfo = None
    childOccInfos: List[ChildOccurrenceInfo] = ()
    cursor: ChildOccsLoadNxtCursor = None


@dataclass
class ChildOccurrencesData(TcBaseObj):
    """
    The information required to get  the child occurrences including list of parent occurrences, input context and flag
    to indicate one level or all levels.
    
    :var parentOccurrences: Occurrences for which children are desired.
    :var inputCtxt: Common input context including filter, configuration, attached data criteria and page size.
    :var firstLevelOnly: If true, only the first level instances are returned.
    :var startFreshNavigation: When set to true Awb0Element with latest information is retrieved. Otherwise existing
    information at Teamcenter server is used.
    :var cloneStableIdChains: List of clone stable id in top-down fashion.  The chain is separated by "/". This field
    is considered only when parentOccurrences is empty.
    """
    parentOccurrences: List[Awb0Element] = ()
    inputCtxt: InputContext = None
    firstLevelOnly: bool = False
    startFreshNavigation: bool = False
    cloneStableIdChains: List[str] = ()


@dataclass
class ChildOccurrencesResp(TcBaseObj):
    """
    The list of child occurrences along with the product context information.
    
    :var output: Vector of child occurrence information for given parents.
    :var serviceData: Service Data
    """
    output: List[ChildOccurrenceInfoList] = ()
    serviceData: ServiceData = None


@dataclass
class InfoForAddElemData(TcBaseObj):
    """
    This structure contains information to identify type that can be added as child of input parentElement or
    productContext.
    
    :var parentElement: Awb0Element under which another Awb0Element to be added as child. This is specified if
    Awb0Element needs to be added under parent context. This is optional parameter.
    :var productContext: Awb0ProductContextInfo of the product in which Awb0Element to be added.
    :var configurationInfo: Configuration information containing RevisionRule, VariantRule, Effectivity information.
    This is used to identify adapter to use for request.
    """
    parentElement: Awb0Element = None
    productContext: Awb0ProductContextInfo = None
    configurationInfo: OccurrenceCnfgInfo = None


@dataclass
class InfoForAddElemResp(TcBaseObj):
    """
    This structure contains allowable child type name, mapped Awb0Element type name and type name to be searched
    through Full Text Search.
    
    :var preferredTypeInfo: Prefrred type information to create Awb0Element.
    :var allowedTypeInfos: List of all allowed types to create and add Awb0Element as child.
    :var preferredExists: true if preferred type is found for input Awb0Element or Awb0ProductContextInfo, false
    otherwise.
    :var serviceData: The service data containing partial error if any.
    """
    preferredTypeInfo: InfoForElementCreation = None
    allowedTypeInfos: List[InfoForElementCreation] = ()
    preferredExists: bool = False
    serviceData: ServiceData = None


@dataclass
class InfoForAddToBookmarkResp(TcBaseObj):
    """
    This structure contains reference to the list of allowable types for creating a product in a Awb0SavedBookmark.
    
    :var allowedTypeInfos: List of all allowed types.
    :var serviceData: The ServiceData containing partial error if any.
    """
    allowedTypeInfos: List[InfoForElementCreation] = ()
    serviceData: ServiceData = None


@dataclass
class InfoForElementCreation(TcBaseObj):
    """
    Object contains allowable business object type name, its mapped Awb0Element type name and type name to be searched
    through Full Text Search.
    
    :var objectTypeName: Awb0Element type name corresponding to objectTypeName.
    :var elementTypeName: Type name for which Awb0Element is instance of in product.
    :var isSearchable: Indicates if the object can be searched through Full Text Search.
    :var searchTypeName: Type name to used to search in Full Text Search. Empty in case if isSearchable is false.
    """
    objectTypeName: str = ''
    elementTypeName: str = ''
    isSearchable: bool = False
    searchTypeName: str = ''


@dataclass
class InputContext(TcBaseObj):
    """
    'InputContext' contains criteria to configure product and retrieve associated data.  This includes occurrence
    filters, configuration info which includes revision and variant configurations product context info, which is
    passed as input from the user if exist. Optionally user can pass StructureContext, which encapsulate product and
    configuration information. If structure context  object given as input then the product and configuration
    information from occurrence configuration is ignored.
    
    :var filter: Specifies a way to filter occurrences in the Product
    :var configuration: Configuration to apply to the product while fetching occurrences.
    :var attachedDataCriteria: Specifies the objects attached to the archetype that need to be returned along with the
    occurrences.
    :var pageSize: Number of occurrences to be fetched per call
    :var structureContextObject: This holds reference to structure context object, which further references
    configuration context and content objects which could be Item, ItemRevision or AppearanceGroup.
    If structureContextObject object is given in input the product and configuration context are optional.
    :var productContext: This indiactes the productContext information, this represent the product along with the
    configuration information.
    :var requestPref: Map of preference names and value pairs (string/string). Allowed preference names are
    includeInterfaces, includeUnconfigured, depthFirst and useGlobalRevRule. Allowed values are true/false. Keys and
    values are case sensitive.
    :var updateAutoBookmark: Update the bookmark state based on the input from this service.
    """
    filter: OccurrenceFilter = None
    configuration: OccurrenceCnfgInfo = None
    attachedDataCriteria: List[AttachedDataCriteria] = ()
    pageSize: int = 0
    structureContextObject: BusinessObject = None
    productContext: Awb0ProductContextInfo = None
    requestPref: RequestPreference = None
    updateAutoBookmark: bool = False


@dataclass
class InsertLevelInputData(TcBaseObj):
    """
    Structure represents the parameters required to insert a parent for the given set of objects.
    
    :var clientId: A unique string supplied by the caller. This is used to identify return data elements and partial
    errors associated with this input structure.
    :var elements: List of Awb0Element objects which the user intends to group and create a new parent for.
    :var object: Business object for which a Awb0Element is to be created and inserted as parent of the given
    Awb0Element objects.
    """
    clientId: str = ''
    elements: List[Awb0Element] = ()
    object: Item = None


@dataclass
class InsertLevelResponse(TcBaseObj):
    """
    Structure represents the output of the operation.
    
    :var output: A list of InsertLevelResponseData structures.
    :var serviceData: The Service Data.
    """
    output: List[InsertLevelResponseData] = ()
    serviceData: ServiceData = None


@dataclass
class InsertLevelResponseData(TcBaseObj):
    """
    Structure represents the output of the operation.
    
    :var clientId: The clientId from the input InsertLevelInputData element. This value is unchanged from the input,
    and can be used to identify this response element with the corresponding input element.
    :var newParent: The Awb0Element  object which is the new inserted parent object of the given set of objects.
    :var newElements: The list of newly created Awb0Element objects as a result of the insertion operation.
    """
    clientId: str = ''
    newParent: Awb0Element = None
    newElements: List[Awb0Element] = ()


@dataclass
class NxtChildOccurrencesData(TcBaseObj):
    """
    The information required to get  the next or previous set of child occurrences of an given parent occurrence(s).
    
    :var cursor: Cursor that represents point to which results have been retrieved
    :var goForward: Direction to navigate to fetch the next page of occurrences
    """
    cursor: ChildOccsLoadNxtCursor = None
    goForward: bool = False


@dataclass
class NxtOccsInProdData(TcBaseObj):
    """
    The 'NxtOccsInProdData' supplies the cursor from a previous call, as input to get the next page of result
    occurrences data. The cursor may have been received from a previous call to either get a first page using
    'getOccurrencesInProduct', or a subsequent page by calling 'getNextOccurrencesInproduct'. The same cursor must be
    passed in as input, as part of 'NxtOccsInProdData' in the subsequent call to  'getNextOccurrencesInproduct'. The
    cursor received from that subsequent call must be passed in to the following invocation to get the following page
    of occurrences, and so on until there is no more data to fetch.
    
    :var cursor: Cursor that represents point to which search results have been retrieved
    :var goForward: Direction to navigate to fetch the next page of occurrences
    """
    cursor: OccsInProdLoadNxtCursor = None
    goForward: bool = False


@dataclass
class OccsInProdLoadNxtCursor(TcBaseObj):
    """
    'OccsInProdLoadNxtCursor' is a cursor that is returned for use as input in calls to 'getNextOccurrencesInProduct'
    to retrieve the next page of search results. This data structure must not be modified by the caller, but passed
    back as is, to the next service call to 'getNextOccurrencesInProduct'. This cursor is received as output inside
    'OccsInProdResp' when a call to 'getNextOccurrencesInProduct' or 'getNextOccurrencesInProduct' is made.
    
    :var contextInfo: The product context info associated with the occurence.
    :var product: End product where the input archetype is used or in context of which the search was performed
    :var inputCtxt: Common input context including filter, configuration, attached data criteria and page size.
    :var firstLevelOnly: Return first level instances only
    :var cursorData: DEPRECATED: Cursor data
    :var startIndex: Cursor start position for the result occurrences returned so far
    :var endIndex: Cursor end position for the result occurrences returned so far
    :var pageSize: This indicate the number of occurrences needs to be returned.
    """
    contextInfo: Awb0ProductContextInfo = None
    product: BusinessObject = None
    inputCtxt: InputContext = None
    firstLevelOnly: bool = False
    cursorData: List[str] = ()
    startIndex: int = 0
    endIndex: int = 0
    pageSize: int = 0


@dataclass
class OccsInProdResp(TcBaseObj):
    """
    'OccsInProdResp' contains the resulting occurrences found for each product from which retrieval was requested. This
    data structure also returns a cursor for each product, which can be sent back to the subsequent call to
    'getNextOccurrencesInProduct' in order to rerieve more pages of results.
    
    :var data: Response data for each input.
    :var serviceData: Service data
    """
    data: List[OccsInProdRespData] = ()
    serviceData: ServiceData = None


@dataclass
class OccsInProdRespData(TcBaseObj):
    """
    'OccsInProdRespData' contains the resulting occurrences found in the configured product, filtered and sorted by the
    criteria requested in the input to the service. This data structure also returns a cursor which can be sent back to
    a call to in order to rerieve more pages of results.
    
    :var productContextInfo: The Occurrence Context Information
    :var occurrences: One page of occurrences in the product.
    :var rootOccurrence: The information on the root occurrence in the product. This occurrence is only returned if all
    occurrences are requested in the product.
    :var cursor: Cursor that represents point to wich the search results have been fetched.
    :var startReached: Reached the first page of occurrences
    :var endReached: Reached the last page of occurrences
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var searchFilterMap: The map containing the list of search filters for each search filter field based on the
    search results.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    """
    productContextInfo: Awb0ProductContextInfo = None
    occurrences: List[OccurrenceInfo] = ()
    rootOccurrence: OccurrenceInfo = None
    cursor: OccsInProdLoadNxtCursor = None
    startReached: bool = False
    endReached: bool = False
    searchFilterCategories: List[SearchFilterField] = ()
    searchFilterMap: SearchFilterMap = None
    defaultFilterFieldDisplayCount: int = 0


@dataclass
class OccurrenceCnfgInfo(TcBaseObj):
    """
    'OccurrenceCnfgInfo' specifies the complete set of configuration rules including RevisionRule objects , Effectivity
    and VariantRule that need to be applied to the input product to configure it in order to retrieve the requested
    occurrence data.
    
    :var revisionRule: Revision Rule
    :var effectivityDate: Effectivity Date to use.
    :var now: Use Now for Effectivity Date Configuration.
    :var endItem: End Item to use for Unit Effectivity
    :var unitNo: Unit Effectivity
    :var variantRule: Variant Rule.
    :var configurationObject: Configuration Object like Configuration Context.  The other configuration information
    above is ignored if this is specified.
    :var svrOwningProduct: This indicates the product ( ItemRevision or CollaborativeDesign ), associated with the
    saved VariantRule.
    """
    revisionRule: RevisionRule = None
    effectivityDate: datetime = None
    now: bool = False
    endItem: BusinessObject = None
    unitNo: int = 0
    variantRule: VariantRule = None
    configurationObject: BusinessObject = None
    svrOwningProduct: BusinessObject = None


class SearchFilterFieldSortType(Enum):
    """
    'SearchFilterFieldSortType' is an enumeration to determine how fields used in filters need to be sorted. The fields
    can be sorted alphabetically in ascending order or in priority order from highest to lowest.
    """
    Alphabetical = 'Alphabetical'
    Priority = 'Priority'


class SearchFilterType(Enum):
    """
    An enumeration for different types of search filters like string, date, numeric.
    """
    StringFilter = 'StringFilter'
    DateFilter = 'DateFilter'
    NumericFilter = 'NumericFilter'


class SortDirection(Enum):
    """
    An enumeration indicating whether the sorting needs to be performed in ascending or descending order.
    """
    ASC = 'ASC'
    DESC = 'DESC'


"""
The map which can have a key and value pair, used for occurrences expansion filters or criteria or options. The key and value are case sensitive. The allowed preference names are 'includeInterfaces, depthFirst'. The allowed values are 'True/False'.
"""
RequestPreference = Dict[str, List[str]]


"""
'SearchFilterMap' is a map containing the list of search filters for each search filter field based on the search results.
"""
SearchFilterMap = Dict[str, List[SearchFilter]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ReportDefinition
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateReportCriteria(TcBaseObj):
    """
    Data needed to generate a report.
    
    :var rdTag: The report definition.
    :var stylesheetTag: The style sheet Dataset . If supplied the style sheet will be applied to the final generated
    report. If this value is left null, a plain XML file will be generated for the report.
    :var contextObjects: A list of context business objects for a Item Report.
    :var datasetName: The name of the containing Dataset. If dataset name is provided, the generated report will be
    saved as a Dataset to Teamcenter with this given name.
    :var criteriaNames: A list of criteria names  in a series of Name/Value pairs used to specify additional criteria
    (optional).
    :var criteriaValues: A list of criteria values in a series of Name/Value pairs used to specify additional criteria
    (optional).
    """
    rdTag: BusinessObject = None
    stylesheetTag: BusinessObject = None
    contextObjects: List[BusinessObject] = ()
    datasetName: str = ''
    criteriaNames: List[str] = ()
    criteriaValues: List[str] = ()


@dataclass
class GenerateReportDefintionIdsResponse(TcBaseObj):
    """
    Response from generateReportDefintionIds.
    
    :var reportdefinitionIds: A list of report definition identifiers.
    :var serviceData: The ServiceData which includes the failure information.
    """
    reportdefinitionIds: List[ReportDefinitionIdObject] = ()
    serviceData: ServiceData = None


@dataclass
class GenerateReportResponse(TcBaseObj):
    """
    Response from generateReport.
    
    :var transientFileReadTickets: This is for future use, may be null.
    :var transientFileTicketInfos: A list of TransientFileTicketInfo  objects for retrieving generated report file. A
    TransientFileTicketInfo object contains the file name, the transient ticket information for the generated report
    file.
    :var serviceData: The ServiceData which includes the failure information.
    """
    transientFileReadTickets: List[str] = ()
    transientFileTicketInfos: List[TransientFileTicketInfo] = ()
    serviceData: ServiceData = None


@dataclass
class GetReportDefinitionsResponse(TcBaseObj):
    """
    Response from getReportDefinitions.
    
    :var reportdefinitions: a vector of ReportDefinitionObjects.
    :var serviceData: A serviceData contains the data.
    """
    reportdefinitions: List[ReportDefinitionObject] = ()
    serviceData: ServiceData = None


@dataclass
class ReportDefinitionIdObject(TcBaseObj):
    """
    A structure associating a report definition id
    with the requesting client.
    
    :var clientId: clientId
    :var reportDefinitionId: reportDefinitionId
    """
    clientId: str = ''
    reportDefinitionId: str = ''


@dataclass
class ReportDefinitionObject(TcBaseObj):
    """
    A structure associating a report definition with the requesting client.
    
    :var clientId: client identifier for routing purposes
    :var reportdefinition: report definition Tag
    """
    clientId: str = ''
    reportdefinition: ReportDefinition = None


@dataclass
class ReportProperties(TcBaseObj):
    """
    Properties needed to create a report definition.
    
    :var rdId: a unique identifier for this definition, may
    be autogenerated or user-specified (required)
    :var rdName: the report name (optional)
    :var rdStylesheets: a vector of stylesheet Tags (optional)
    :var rdEntries: a vector of strings containing the Entries in a series of
    Entry/Value pairs used to specify additional metadata (optional)
    :var rdValues: a vector of strings containing the Values in a series of
    Entry/Value pairs used to specify additional metadata (optional)
    :var rdDescription: the report description (optional)
    :var rdType: integer report type identifier, e.g. item, summary or custom (required)
    :var rdClass: class of designated report type (required for item reports)
    :var rdFolderName: name of the containing folder for this report definition (optional)
    :var rdSource: solution source of this definition, e.g. Teamcenter, TcRA (optional)
    :var rdState: integer for future use, may be any value
    :var rdQuerySource: the source of the report data (required)
    :var rdPropertySource: properties to include in the report (optional)
    """
    rdId: str = ''
    rdName: str = ''
    rdStylesheets: List[BusinessObject] = ()
    rdEntries: List[str] = ()
    rdValues: List[str] = ()
    rdDescription: str = ''
    rdType: int = 0
    rdClass: str = ''
    rdFolderName: str = ''
    rdSource: str = ''
    rdState: int = 0
    rdQuerySource: str = ''
    rdPropertySource: str = ''


@dataclass
class ReportsCriteria(TcBaseObj):
    """
    Criteria needed to retrieve report definitions
    or generate report definition ids. At least one
    of the optional parameters must be included.
    
    :var clientId: Client identifier for routing purposes (required).
    :var category: Designates report category, e.g. item, summary or custom (optional).
    :var source: Solution source of report definition, e.g. Teamcenter, TcRA (optional).
    :var status: It is for future use, may be null.
    :var contextObjects: A vector of gags representing context object(s) (required for item reports).
    """
    clientId: str = ''
    category: str = ''
    source: str = ''
    status: str = ''
    contextObjects: List[BusinessObject] = ()


@dataclass
class TransientFileInfo(TcBaseObj):
    """
    The information required for file upload.
    
    :var fileName: Absolute file path and name to transient volume
    :var isText: True if filetype is text
    """
    fileName: str = ''
    isText: bool = False


@dataclass
class TransientFileTicketInfo(TcBaseObj):
    """
    TransientFileInfo with a ticket.
    
    :var transientFileInfo: Information on the file to be downloaded.
    :var ticket: FMS file ticket
    """
    transientFileInfo: TransientFileInfo = None
    ticket: str = ''


@dataclass
class CreateReportDefinitionResponse(TcBaseObj):
    """
    Response from createReportDefinition.
    
    :var reportdefinition: report definition Tag
    :var serviceData: ServiceData
    """
    reportdefinition: BusinessObject = None
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ReportDefinition
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class GenerateReportsCriteria2(TcBaseObj):
    """
    The criteria object needed to generate report.
    
    :var clientId: The client identifier for routing purposes (required).
    :var rdTag: The Report Definition ID.
    :var datasetCtxUID: The uid for the context Dataset.
    :var datasetCtxObj: The Dataset context ID.
    :var relationName: The relation name to be used.
    :var datasetType: The Dataset type to be used.
    :var reportOptionsNames: A list of strings containing the Names in a series of Name/Value pairs used to specify
    additional criteria (optional).
    :var reportOptionsValues: A list of strings containing the Names in a series of Name/Value pairs used to specify
    additional criteria (optional).
    :var reportName: Designates the name of the report given the rdTag is not supplied.
    :var stylesheetTag: The report style ID (optional).
    :var stylesheetName: The name of the report style if known
    :var contextObjects: A list of ID's representing context object(s) (required for item reports).
    :var contextObjectUIDs: A list of uids representing context objects.
    :var datasetName: The name of containing DataSet (optional).
    :var criteriaNames: A list of strings containing the Names in a series of Name/Value pairs used to specify criteria
    for saved queries(optional).
    :var criteriaValues: A list of strings containing the Values in a series of Name/Value pairs used to specify
    criteria for saved queries(optional).
    """
    clientId: str = ''
    rdTag: ReportDefinition = None
    datasetCtxUID: str = ''
    datasetCtxObj: BusinessObject = None
    relationName: str = ''
    datasetType: str = ''
    reportOptionsNames: List[str] = ()
    reportOptionsValues: List[str] = ()
    reportName: str = ''
    stylesheetTag: BusinessObject = None
    stylesheetName: str = ''
    contextObjects: List[BusinessObject] = ()
    contextObjectUIDs: List[str] = ()
    datasetName: str = ''
    criteriaNames: List[str] = ()
    criteriaValues: List[str] = ()



from __future__ import annotations

from tcsoa.gen.Ai._2009_10.Ai import GetPropertyValuesResponse, GetPropertyValuesData
from typing import List
from tcsoa.utils import TcService


class AiService(TcService):

    @classmethod
    def getPropertyValues(cls, input: List[GetPropertyValuesData]) -> GetPropertyValuesResponse:
        """
        get the property values for the object supplied as ApplicationReferences and configuration.
        """
        return cls.execute_soa_method(
            method_name='getPropertyValues',
            library='Ai',
            service_date='2009_10',
            service_name='Ai',
            params={'input': input},
            response_cls=GetPropertyValuesResponse,
        )



from __future__ import annotations

from tcsoa.gen.StructureManagement._2013_05.VariantManagement import SetBOMVariantRulesResponse, InputBOMLinesSet, GetVariantExpressionsMatchInfoResponse, BOMVariantRuleContents, SetBOMVariantRuleData, GetBOMVariantRuleInput, ApplyBOMVariantRulesResponse, BOMVariantRulesResponse
from tcsoa.gen.StructureManagement._2013_05.StructureVerification import AttributeGroupAndFormComparisonResponse, ConnectedObjectsComparisonResponse
from tcsoa.gen.BusinessObjects import BOMWindow
from tcsoa.gen.StructureManagement._2012_02.StructureVerification import EquivalentLines
from typing import List
from tcsoa.gen.StructureManagement._2013_05.IncrementalChange import ObjectsICEInfo
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class VariantManagementService(TcService):

    @classmethod
    def getVariantExpressionsMatchInfo(cls, inputBOMLinesSets: List[InputBOMLinesSet]) -> GetVariantExpressionsMatchInfoResponse:
        """
        This operation calculates and returns the rolledup variant expressions and rolledUp clause lists for the input
        rollUpBomLines. For the nonRolledUpBomLines, BOMLine variant conditions and clause lists will be returned. . If
        doCompare parameter is set as true, then this operation compares the equivalent lines based on the variant
        conditions and sets the isDifferent variable accordingly. The lines in input rollUpBomLines, will be compared
        using the rolled up variants and the lines in the nonRollUpBomLines list will be compared using variant
        conditions. All the lines in one InputBOMLinesSet will be compared with each other till a difference is found.
        """
        return cls.execute_soa_method(
            method_name='getVariantExpressionsMatchInfo',
            library='StructureManagement',
            service_date='2013_05',
            service_name='VariantManagement',
            params={'inputBOMLinesSets': inputBOMLinesSets},
            response_cls=GetVariantExpressionsMatchInfoResponse,
        )

    @classmethod
    def setBOMVariantRules(cls, setBOMVariantRuleInput: List[SetBOMVariantRuleData]) -> SetBOMVariantRulesResponse:
        """
        This operation set the input variant rule to the window and returns the list of variant rule and respective
        window.
        
        Use cases:
        This operation should be used when user want to set values of the option on variant rule.
        
        Exceptions:
        >This operation may raise a Teamcenter::Soa::Server::ServiceException wrapping around following Teamcenter
        errors:
        
        46001 Invalid  input is received from input structure SetBOMVariantRuleData
        """
        return cls.execute_soa_method(
            method_name='setBOMVariantRules',
            library='StructureManagement',
            service_date='2013_05',
            service_name='VariantManagement',
            params={'setBOMVariantRuleInput': setBOMVariantRuleInput},
            response_cls=SetBOMVariantRulesResponse,
        )

    @classmethod
    def applyBOMVariantRules(cls, window: BOMWindow, rules: List[BOMVariantRuleContents]) -> ApplyBOMVariantRulesResponse:
        """
        The applyBOMVariantRules operation applies either given BOM variant rules or Saved Variant Rules to the window.
        BOM Variant rules that contain options having multiple values can be applied to the window. Output will be the
        window and list of BOM variant rules and Saved Variant Rules have been applied to the window.
        
        Use cases:
        This operation will be used when BOM variant rules or Saved Variant Rules needs to be applied on the window.
        
        Exceptions:
        >This operation may raise a Teamcenter::Soa::Server::ServiceException wrapping around following Teamcenter
        errors:
        - 46001 Invalid input  is received from input structure BomVariantRuleContents.
        - 214507 Single variant rule cannot be applied if window is in Overlay mode. Multiple variant rule is expected.
        - 214555 Only provide either BOM Variant Rules or Saved Variant Rules.
        
        """
        return cls.execute_soa_method(
            method_name='applyBOMVariantRules',
            library='StructureManagement',
            service_date='2013_05',
            service_name='VariantManagement',
            params={'window': window, 'rules': rules},
            response_cls=ApplyBOMVariantRulesResponse,
        )

    @classmethod
    def getBOMVariantRules(cls, variantRuleInput: List[GetBOMVariantRuleInput]) -> BOMVariantRulesResponse:
        """
        This operation takes list of window and its identifier and returns variant rules and saved variant rules
        associated with the window. As part of input in this operation user can provide saved variant rule along with
        saved variant rule action mode. This action indicates add, remove, update or override actions related to saved
        variant rule. There could be multiple variant rules, associated with window. List of these rules will be
        returned as the output. It also returns list of option and list of values associated with each option.  A flag
        in the value list indicates, if window is configured with the particular option value.  There could be multiple
        values associated with a single option and there could be multiple saved variant rules associated with a window.
        
        Use cases:
        This operation should be used when user wants to get variant rules associated with window. User may also use it
        to set , unset, override or update saved variant rule based on the window mode
        
        Exceptions:
        >This operation may raise a Teamcenter::Soa::Server::ServiceException wrapping around following Teamcenter
        errors.
        - 46001     Invalid input is received from input structure GetBomVariantRuleInput 
        - 214508   Overlay requires at least two variant rules to be loaded.   You cannot remove input variant rule.
        - 214509   Input Variant rule is invalid for the current bom window mode.
        - 214510   For the input svrActionMode, input saved variant rules are invalid. 
        
        """
        return cls.execute_soa_method(
            method_name='getBOMVariantRules',
            library='StructureManagement',
            service_date='2013_05',
            service_name='VariantManagement',
            params={'variantRuleInput': variantRuleInput},
            response_cls=BOMVariantRulesResponse,
        )


class IncrementalChangeService(TcService):

    @classmethod
    def removeIncrementalChanges(cls, listOfObjects: List[ObjectsICEInfo]) -> ServiceData:
        """
        This operation deletes the list of Incremental Change Elements (ICE) on the given BOMLine or the MECfgLine.
        """
        return cls.execute_soa_method(
            method_name='removeIncrementalChanges',
            library='StructureManagement',
            service_date='2013_05',
            service_name='IncrementalChange',
            params={'listOfObjects': listOfObjects},
            response_cls=ServiceData,
        )


class StructureVerificationService(TcService):

    @classmethod
    def getAttributeGroupsAndFormsComparisonDetails(cls, equivalentObjects: List[EquivalentLines], attributeGroupsNames: List[str]) -> AttributeGroupAndFormComparisonResponse:
        """
        This operation returns the details of differences between the supplied Attribute Groups for the supplied
        equivalent objects (that can be Cpd0DesignElement, Cpd0DesignFeature, or BOMLine objects). For each supplied
        attribute group the operation returns the list of its attributes, the attributes values for each supplied
        source and target, and the result of comparing each attribute on all supplied sources and targets.
        """
        return cls.execute_soa_method(
            method_name='getAttributeGroupsAndFormsComparisonDetails',
            library='StructureManagement',
            service_date='2013_05',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects, 'attributeGroupsNames': attributeGroupsNames},
            response_cls=AttributeGroupAndFormComparisonResponse,
        )

    @classmethod
    def getConnectedObjectsComparisonDetails(cls, equivalentObjects: List[EquivalentLines]) -> ConnectedObjectsComparisonResponse:
        """
        This operation returns the details of any differences between connected objects (that can be either BOMLines or
        Cpd0DesignElements) for the supplied equivalent objects (that can be either BOMLines or Cpd0DesignFeatures).
        The operation takes the source and target and compares their connected objects. The source and target connected
        objects are returned by this operation in the form of a table that is created by the output structures.
        """
        return cls.execute_soa_method(
            method_name='getConnectedObjectsComparisonDetails',
            library='StructureManagement',
            service_date='2013_05',
            service_name='StructureVerification',
            params={'equivalentObjects': equivalentObjects},
            response_cls=ConnectedObjectsComparisonResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ClassificationObjectResponse(TcBaseObj):
    """
    Holds information on created, updated or deleted classification objects.
    
    :var jsonResponse: This is a JSON string containing information about the created, updated, retrieved or deleted
    classification objects.
    Any failures occurred during the operation will be specified in the JSON string under the 'ErrorDetails' container.
    The response structure will follow the JSON schema defined in the file:
    TC_DATA\classification\json\1.0.0\schema\Response.schema.json.
    Refer the following sample responses for operations on classification objects:
    Save and Retrieve:- TC_DATA\classification\json\1.0.0\samples\GetClassificationObjects_Response_sample.json
    Delete:- TC_DATA\classification\json\1.0.0\samples\DeleteClassificationObjects_Response_sample.json
    Error:- TC_DATA\classification\json\1.0.0\samples\SaveClassificationObjects_Error_Response_sample.json
    :var serviceData: This is a placeholder for future use. All information about the created, updated and deleted
    classification objects and any errors, occurred during the operation, will be returned in the JSON response string.
    """
    jsonResponse: str = ''
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Cad._2009_04.StructureManagement import CreateOrUpdateRelativeStructureInfo, CreateOrUpdateRelativeStructurePref, CreateOrUpdateRelativeStructureResponse
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def createOrUpdateRelativeStructure(cls, inputs: List[CreateOrUpdateRelativeStructureInfo], pref: CreateOrUpdateRelativeStructurePref) -> CreateOrUpdateRelativeStructureResponse:
        """
        This operation creates or updates the relative structure for the input parent assembly and child components. 
        The objects created or updated by this operation include a BOM view (BV), BOM view revision (BVR) and
        occurrence data (PSOccurrence, PSOccurrenceThread).
         
        Before creating the relative structure objects and relations, this operation will check that the structure to
        be created does not already exist.  If the occurrence exists but the input attribute values differ from those
        already set, an attempt is made to update the values.
        
        This operation assumes the input is in a bottom up format such that if any failures occur, the structure that
        is created will still be consumable.  For example:
        
        Parent assembly A, subassembly B and child C are input into this operation.  If the relative structure for
        subassembly B and child C is created successfully but an error occurs during the creation of the structure for
        assembly A and subassembly B, the client can still access the subassembly B, child C relative structure.
        
        No attempt is made in the operation to rearrange the input and process it in the correct order. One parent
        context object is processed at one time and it is assumed that all edits for a given parent context come in as
        one set of input. 
        
        
        Use cases:
        Use Case 1:
        
        User adds an existing component to an existing assembly to create a relative occurrence.
        For this operation, the assembly is passed in as the parent and the component is passed in as the child.  The
        relative occurrence is created and a map of the input 'clientID' to 'MappedReturnData' is returned in 'output'
        and the occurrence, BOM view and BOM view revision are returned as created objects in 'ServiceData'.
        
        Use Case 2:
        
        User wants to update the position of the child component relative to the parent assembly.
        For this operation, the transform on the child occurrence information is updated with the new position relative
        to the parent.  The assembly is passed in as the parent and the component is passed in as the child.  The
        relative occurrence is updated and a map of the 'clientID' to 'MappedReturnData' is returned in 'output' and
        the occurrence and BOM view revision are returned as updated objects in 'ServiceData'.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRelativeStructure',
            library='Cad',
            service_date='2009_04',
            service_name='StructureManagement',
            params={'inputs': inputs, 'pref': pref},
            response_cls=CreateOrUpdateRelativeStructureResponse,
        )



from tcsoa.gen.Internal_ChangeManagement._2015_03.services import ChangeManagementService as imp0
from tcsoa.gen.Internal_ChangeManagement._2012_10.services import ChangeManagementService as imp1
from tcsoa.gen.Internal_ChangeManagement._2020_01.services import MassUpdateService as imp2
from tcsoa.utils import TcService


class ChangeManagementService(TcService):
    connectChangeNoticeToContext = imp0.connectChangeNoticeToContext
    createOrUpdatePreviousEffectivity = imp0.createOrUpdatePreviousEffectivity
    disconnectChangeNoticeFromContext = imp0.disconnectChangeNoticeFromContext
    getCreatableChangeTypes = imp1.getCreatableChangeTypes
    removePrevEffectivityFromChgNotice = imp0.removePrevEffectivityFromChgNotice
    updateChangeNoticeRelations = imp0.updateChangeNoticeRelations


class MassUpdateService(TcService):
    hasActiveMarkupAssociated = imp2.hasActiveMarkupAssociated
    saveImpactedAssemblies = imp2.saveImpactedAssemblies



from __future__ import annotations

from tcsoa.gen.Cad._2008_03.StructureManagement import AskChildPathBOMLinesInfo
from tcsoa.gen.Cad._2013_05.StructureManagement import AskChildPathBOMLinesResponse2, CreateOrUpdateRelativeStructureInfo4, CreateWindowsInfo2
from tcsoa.gen.Cad._2007_01.StructureManagement import CreateBOMWindowsResponse
from tcsoa.gen.Cad._2009_04.StructureManagement import CreateOrUpdateRelativeStructurePref, CreateOrUpdateRelativeStructureResponse
from typing import List
from tcsoa.utils import TcService


class StructureManagementService(TcService):

    @classmethod
    def createBOMWindows2(cls, info: List[CreateWindowsInfo2]) -> CreateBOMWindowsResponse:
        """
        Creates a list of window and sets the respective input ItemRevision as the top line. This operation can be used
        to set multiple saved variant rules or single stored option set to the window. For setting Product Configurator
        authored varaint rule on the window, value of preference PSM_enable_product_configurator must be true.It can be
        used to set certain window property, if sent as a part of input. It can be used to create the BOMLine for input
        to Expand Product Structure services.  All BOMLines under this window are unpacked.  To use the Teamcenter
        default unitNo or use your input RevisionRule with no changes, you must set unitNo to -1 in
        RevisionRuleEntryProps::unitNo.  If it is not specified, your input rule will function as a modified/transient
        revision rule with a unitNo of 0.
        
        Use cases:
        This operation creates a list of window and sets the respective input Item revision as the top line. This
        operation can be used to set multiple saved variant rules or single stored option set to the window.
        """
        return cls.execute_soa_method(
            method_name='createBOMWindows2',
            library='Cad',
            service_date='2013_05',
            service_name='StructureManagement',
            params={'info': info},
            response_cls=CreateBOMWindowsResponse,
        )

    @classmethod
    def createOrUpdateRelativeStructure(cls, inputs: List[CreateOrUpdateRelativeStructureInfo4], pref: CreateOrUpdateRelativeStructurePref) -> CreateOrUpdateRelativeStructureResponse:
        """
        This operation creates or updates the relative structure for the input parent assembly and child components. 
        The objects created or updated by this operation include a BOM view (BV), BOM view revision (BVR) and
        occurrence data (PSOccurrence, PSOccurrenceThread).
         
        Before creating the relative structure objects and relations, this operation will check that the structure to
        be created does not already exist.  If the occurrence exists but the input attribute values differ from those
        already set, an attempt is made to update the values.
        
        This operation assumes the input is in a bottom up format such that if any failures occur, the structure that
        is created will still be consumable.  For example:
        
        Parent assembly A, subassembly B and child C are input into this operation.  If the relative structure for
        subassembly B and child C is created successfully but an error occurs during the creation of the structure for
        assembly A and subassembly B, the client can still access the subassembly B, child C relative structure.
        
        No attempt is made in the operation to rearrange the input and process it in the correct order. One parent
        context object is processed at one time and it is assumed that all edits for a given parent context come in as
        one set of input.
        
        This operation can also perform structure move operations. The move operation is performed within the context
        of the lowest common parent. Along with moving the occurrence from one parent to a new parent, any absolute
        occurrence data present in the original occurrence will also be moved to the new occurrence. The input for the
        move operation is specified in the 'MoveInfo' substructure of 'RelOccInfo'. The 'MoveInfo' data is specified in
        the context of the 'RelativeStructureChildInfo3'. Each 'RelativeStructureChildInfo3' will allow specifying only
        moving of one instance only. In cases where there are multiple instances of the same occurrence, 'MoveInfo' has
        to be specified for each instance. In such cases, each instance data will be specified in its own
        'RelativeStructureChildInfo3' structure. However, the 'child' object is specified only once per occurrence. For
        the other instances of the same occurrence, the 'child' object in the 'RelativeStructureChildInfo3' is
        specified as NULL.
        
        In the following example, to move Part_P1 (instance 1) to SubAssembly_B (instance 1), create a new occurrence
        of Part_P1 under SubAssembly_B. After this add, SubAssembly_B (instance 1) will have a child  Part_P1 (instance
        1) and SubAssembly_B (instance 2) will have a child  Part_P1 (instance 2). Specify the move from the occurrence
        of Part_P1 (instance 1) under SubAssembly_C to the Part_P1 (instance 1) under SubAssembly_B (instance 1).  And
        similarly for the instance 2.
        
        Move Example:
        Assembly_A
             |
             |-->SubAssembly_B  (instance 1)
             |
             |-->SubAssembly_B  (instance 2)
             |
             |-->SubAssembly_C
             |       |
             |       |-->Part_P1    (instance 1)
             | 
             |-->SubAssembly_C
             |       |
             |       |-->Part_P1    (instance 2)
        
        
        To move "Part_P1" from SubAssembly_C to Sub_Assembly_B:
        'relStrInfo4s''[''0''] = new ''CreateOrUpdateRelativeStructureInfo4''();
        ''relStrInfo4s''[''0''].''parentInfo'' = new ''RelativeStructureParentInfo''();
        ''relStrInfo4s''[''0''].''parentInfo.parent'' = ''SubAssembly_B_ItemRev'';
        ''relStrInfo4s''[''0''].''childInfo'' = new ''RelativeStructureChildInfo3''[''1''];
        
        ''relStrInfo4s''[''0''].''childInfo''[''0''] = new ''RelativeStructureChildInfo3''();
        ''relStrInfo4s''[''0''].''childInfo''[''0''].''clientId'' = "''Add_Part_P1_to_SubAssembly_B''";
        ''relStrInfo4s''[''0''].''childInfo''[''0''].child = ''Part_P1'';
        ''relStrInfo4s''[''0''].''childInfo''[''0''].''occInfo'' = new ''RelOccInfo''();
        ''relStrInfo4s''[''0''].''childInfo''[''0''].''occInfo.moveInfo'' = new ''MoveInfo''();
        ''relStrInfo4s''[''0''].''childInfo''[''0''].''occInfo.moveInfo.commonParent'' = ''Assembly_A_ItemRev'';
        ''relStrInfo4s''[''0''].''childInfo''[''0''].''occInfo.moveInfo.sourceAssembly'' = ''SubAssembly_C_ItemRev'';
        ''relStrInfo4s''[''0''].''childInfo''[''0''].''occInfo.moveInfo.occThreadPathToBeMoved'' =
        ''occThreadPathToBeMoved'';
        ''relStrInfo4s''[''0''].''childInfo''[''0''].''occInfo.moveInfo.occThreadPathTargetParent'' =
        ''targetParentOccThr'';'
        
        relStrInfo4s[0].childInfo[1] = new RelativeStructureChildInfo3();
        relStrInfo4s[0].childInfo[1].clientId = "Add_Part_P1_instance_2_to_SubAssembly_B";
        relStrInfo4s[0].childInfo[1].child = NULL;   // For the second instance, set child to NULL
        relStrInfo4s[0].childInfo[1].occInfo = new RelOccInfo();
        relStrInfo4s[0].childInfo[1].occInfo.moveInfo = new MoveInfo();
        relStrInfo4s[0].childInfo[1].occInfo.moveInfo.commonParent = Assembly_A_ItemRev;
        relStrInfo4s[0].childInfo[1].occInfo.moveInfo.sourceAssembly = SubAssembly_C_ItemRev;
        relStrInfo4s[0].childInfo[1].occInfo.moveInfo.occThreadPathToBeMoved = occThreadPathToBeMoved;
        relStrInfo4s[0].childInfo[1].occInfo.moveInfo.occThreadPathTargetParent = targetParentOccThr;
        
        
        Use cases:
        Use Case 1:
        
        User adds an existing component to an existing assembly to create a relative occurrence.
        For this operation, the assembly is passed in as the parent and the component is passed in as the child. The
        relative occurrence is created and a map of the input 'clientID' to 'MappedReturnData' is returned in output
        and the occurrence, BOM view and BOM view revision are returned as created objects in 'ServiceData'.
        
        Use Case 2:
        
        User wants to update the position of the child component relative to the parent assembly.
        For this operation, the transform on the child occurrence information is updated with the new position relative
        to the parent. The assembly is passed in as the parent and the component is passed in as the child. The
        relative occurrence is updated and a map of the 'clientID' to 'MappedReturnData' is returned in 'output' and
        the occurrence and BOM view revision are returned as updated objects in 'ServiceData'.
        
        Exceptions:
        >Service exceptions are thrown for the following error situations:
        
        215005: The BOM view revision (BVR) is not checked out by the current user.
        215006: The BOM view revision is checked out by another user.
        215009: The object type name specified in the input preference does not exist.
        215010: The input CAD occurrence identifier does not correspond to a valid BOM line. This is an error when the
        complete input is false.
        215033: The input last modified date differs from actual.
        215034: The dataset or BVR was modified even when the input last modified dates was different than the current
        last modified data.
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRelativeStructure',
            library='Cad',
            service_date='2013_05',
            service_name='StructureManagement',
            params={'inputs': inputs, 'pref': pref},
            response_cls=CreateOrUpdateRelativeStructureResponse,
        )

    @classmethod
    def askChildPathBOMLines2(cls, input: List[AskChildPathBOMLinesInfo]) -> AskChildPathBOMLinesResponse2:
        """
        This operation returns the BOMLine objects corresponding to input sets of child paths.
        
        The child path is defined by an ordered set of PSOccurrenceThread UIDs, starting from the top level assembly to
        a child node. The child node can be an immediate child or can be multi-level deep. 
        
        This operation will return the BOMLine for each input child path. If a particular child path no longer exists
        in the Teamcenter product structure, there will be no entry for that input child path in the output
        'AskChildPathClientIdToBOMLineMap'.
        
        This operation works on existing BOM windows. The BOM window must have been created prior to using this
        operation.
        
        Use cases:
        If the client has a previously saved list of PSOccurrenceThread paths of a product structure, a new BOM window
        for the top level Item can be created and all the BOM line objects corresponding to the leaf node of the
        PSOccurrenceThread paths can be queried through this operation.
        """
        return cls.execute_soa_method(
            method_name='askChildPathBOMLines2',
            library='Cad',
            service_date='2013_05',
            service_name='StructureManagement',
            params={'input': input},
            response_cls=AskChildPathBOMLinesResponse2,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, IncrementalChangeElement
from typing import List
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class ObjectsICEInfo(TcBaseObj):
    """
    A structure to hold BOMLine or the MECfgLine component and their corresponding ICE.
    
    :var object: Business object (e.g. BOMLine, MECfgLine) for which the list of ICE are mapped with.
    
    :var ices: The list of structural ICE elements for the given business object.
    """
    object: BusinessObject = None
    ices: List[IncrementalChangeElement] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, ImanFile
from tcsoa.gen.Internal.Core._2008_06.FileManagement import CommitUploadedRegularFilesInput, GetFileTransferTicketsResponse, FileTicketsResponse, GetRegularFileWriteTicketsResponse, CommitUploadedRegularFilesResponse, UpdateImanFileCommitsResponse, WriteTicketsInput, GetRegularFileWriteTicketsInput
from tcsoa.gen.Internal.Core._2008_06.DispatcherManagement import InsertDatasetVersionArgs, QueryDispatcherRequestsResponse, CreateDatasetOfVersionArgs, CreateDatasetOfVersionResponse, InsertDatasetVersionResponse, UpdateDispatcherRequestResponse, UpdateDispatcherRequestArgs, QueryDispatcherRequestsArgs
from tcsoa.gen.Internal.Core._2008_06.DataManagement import MultiRelationMultiLevelExpandResponse, CreateRelationsResponse, MultiRelMultiLevelExpandInput, Relationship
from typing import List
from tcsoa.gen.Core._2008_06.DataManagement import SaveAsNewItemResponse2, SaveAsNewItemInfo, ReviseInfo, ReviseResponse2
from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcService


class FileManagementService(TcService):

    @classmethod
    def getRegularFileTicketsForUpload(cls, inputs: List[GetRegularFileWriteTicketsInput]) -> GetRegularFileWriteTicketsResponse:
        """
        This operation obtains File Management System (FMS) write tickets for a set of supplied ImanFile objects.  The
        supplied tickets are required by the File Management System (FMS) to upload files to a Teamcenter volume.  The
        'imanFiles' input parameter contains the list of ImanFile objects to be transferred.
        These files are typically standalone files which are not attached to a Dataset as named reference.   No Dataset
        or named reference relation is required or created during this operation.  This operation is intended for use
        only by the Dispatcher module to store input and log files in Teamcenter volumes.
        This operation supports the upload (addition) of files to a Teamcenter volume.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        
        Use cases:
        Two major use cases are supported:
        - Attach translation input files to a Dispatcher request during request creation.
        
        
        In this use case, a Dispatcher Client uploads the input files to a Teamcenter volume, so that the ImanFile
        representing the data file can be attached to a Dispatcher request.
        - Attach log files to Dispatcher request during request update.
        
        
        In this use case, the Dispatcher Client uploads the log file of a completed translation task to a Teamcenter
        volume, and attaches a reference to the ImanFile representing the log file to the Dispatcher request for post
        mortem analysis.
        """
        return cls.execute_soa_method(
            method_name='getRegularFileTicketsForUpload',
            library='Internal-Core',
            service_date='2008_06',
            service_name='FileManagement',
            params={'inputs': inputs},
            response_cls=GetRegularFileWriteTicketsResponse,
        )

    @classmethod
    def getWriteTickets(cls, inputs: List[WriteTicketsInput]) -> FileTicketsResponse:
        """
        This operation obtains File Management System (FMS) write tickets  for a set of supplied files that will be
        uploaded to Teamcenter.  Dataset creation is not required prior to requesting the tickets. The write tickets
        are used to transfer files from a local storage location to the Teamcenter volume.
        This operation is used in conjunction with other 'FileManagementService' operations , Visualization operations
        for creating the datamodel , committing the uploaded file to the Teamcenter database and associating the
        uploaded data files in the Teamcenter volume with the Dataset named references and the File Management System
        (FMS).  Please consult the documentation for each of these available operations for details on the
        requirements, usage, and environments in which they should be used.
        This operation supports the upload (addition) of files as ImanFile objectsto a Teamcenter volume.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        
        Use cases:
        This operation supports file uploads from the Teamcenter Visualization module.
        """
        return cls.execute_soa_method(
            method_name='getWriteTickets',
            library='Internal-Core',
            service_date='2008_06',
            service_name='FileManagement',
            params={'inputs': inputs},
            response_cls=FileTicketsResponse,
        )

    @classmethod
    def commitRegularFiles(cls, inputs: List[CommitUploadedRegularFilesInput]) -> CommitUploadedRegularFilesResponse:
        """
        This operation is invoked after successfully uploading a file to a Teamcenter volume.
        These files are typically standalone files which are not attached to a Dataset as named reference.   No Dataset
        or named reference relation is required or created during this operation.  This operation is intended for use
        only by the Dispatcher module to store input and log files in Teamcenter volumes.
        This operation supports the upload (addition) of files to a Teamcenter volume.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        
        Use cases:
        Two major use cases are supported:
        - Attach translation input files to a Dispatcher request during request creation.
        
        
        In this use case, a Dispatcher Client uploads the input files to a Teamcenter volume, so that the ImanFile
        representing the data file can be attached to a Dispatcher request.
        - Attach log files to Dispatcher request during request update.
        
        
        In this use case, the Dispatcher Client uploads the log file of a completed translation task to a Teamcenter
        volume, and attaches a reference to the ImanFile representing the log file to the Dispatcher request for
        postmortem analysis.
        """
        return cls.execute_soa_method(
            method_name='commitRegularFiles',
            library='Internal-Core',
            service_date='2008_06',
            service_name='FileManagement',
            params={'inputs': inputs},
            response_cls=CommitUploadedRegularFilesResponse,
        )

    @classmethod
    def updateImanFileCommits(cls, cleanupInfo: List[str]) -> UpdateImanFileCommitsResponse:
        """
        This operation recommits files that have been successfully moved to a new volume by the File Management System
        (FMS) StoreAndForward feature.  This operation is intended for use only by the 'DatabaseOperations'() method of
        an FMSTransferTool transfer task running in a Dispatcher Client context.
        The File Management System (FMS) StoreAndForward feature redirects uploads to a temporary volume that is close
        to the uploading user, called a Default Local Volume.  The system then transfers the file to its final home in
        a Default Volume in a background FMSTransferTool process executed on a Dispatcher Module.
        The first step of the background operation is to transfer the file(s) from the Default Local Volume to the
        Default Volume.  After the file transfer completes, the DatabaseOperations step of the FMSFileTransferTool
        dispatcher task invokes this operation to commit the new volumes of each file to the Teamcenter database.
        This operation also returns the write tickets necessary to remove the file from the Default Local Volume after
        any outstanding FMS tickets expire.  This information is persisted in a second dispatcher task, which also
        executes in a background operation on a Dispatcher Module.
        The FMSTransferTool TaskPrep method invokes the 'getFileTransferTickets'() operation to obtain the FMS tickets
        necessary to transfer the files between the two volumes.  The tickets are written to a temporary file stored in
        the Dispatcher staging directory, and processed by the FMSTransferTool.  The FMSTransferTool manages the file
        transfers, and writes the results to an output file.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        
        Use cases:
        This operation supports the File Management System (FMS) StoreAndForward feature.
        """
        return cls.execute_soa_method(
            method_name='updateImanFileCommits',
            library='Internal-Core',
            service_date='2008_06',
            service_name='FileManagement',
            params={'cleanupInfo': cleanupInfo},
            response_cls=UpdateImanFileCommitsResponse,
        )

    @classmethod
    def getFileTransferTickets(cls, imanFiles: List[ImanFile]) -> GetFileTransferTicketsResponse:
        """
        This operation obtains File Management System (FMS) transfer tickets for a set of supplied ImanFile objects. 
        The supplied tickets are used to transfer files from a temporary volume to a final destination volume as a part
        of the FMS StoreAndForward feature.  The imanFiles input parameter contains the list of ImanFile objects to be
        transferred.  This operation is intended for use only by the TaskPrep method of an FMSTransferTool transfer
        task running in a Dispatcher Client context.
        The File Management System (FMS) StoreAndForward feature redirects uploads to a temporary volume that is close
        to the uploading user, called a Default Local Volume.  The system then transfers the file to its final home in
        a Default Volume in a background FMSTransferTool  process executed on a Dispatcher Module.
        The first step of the background operation is to transfer the file(s) from the Default Local Volume to the
        Default Volume.  This operation is invoked by the FMSTransferTool TaskPrep() method to obtain the FMS tickets
        necessary to transfer the files between the two volumes.  The tickets are written to a temporary file stored in
        the Dispatcher staging directory, and processed by the FMSTransferTool.  The FMSTransferTool manages the file
        transfers, and writes the results to an output file.
        After the files are successfully transferred from the Default Local Volume to the Default Volume, the
        'DatabaseOperations'() method of the FMSFileTransferTool dispatcher task invokes the
        'FileManagementService::updateImanFileCommits' operation to update the Teamcenter database with the new file
        location.
        This operation is unpublished.  It is supported only for internal Siemens PLM purposes.  Customers should not
        invoke this operation.
        
        Use cases:
        This operation supports the File Management System (FMS) StoreAndForward feature.
        """
        return cls.execute_soa_method(
            method_name='getFileTransferTickets',
            library='Internal-Core',
            service_date='2008_06',
            service_name='FileManagement',
            params={'imanFiles': imanFiles},
            response_cls=GetFileTransferTicketsResponse,
        )


class DispatcherManagementService(TcService):

    @classmethod
    def insertDatasetVersion(cls, inputs: List[InsertDatasetVersionArgs]) -> InsertDatasetVersionResponse:
        """
        Insert a Datasets at a specific version.
        
        Use cases:
        When creating Datasets in the system, sometimes the versions are to be specified.  This is an important detail
        to maintain, especially during migration of external data into Teamcenter from another system.  This operation
        provides the ability to insert a Dataset at a particular version within the system which are different than the
        standard behavior provided by the Data Management Services.
        """
        return cls.execute_soa_method(
            method_name='insertDatasetVersion',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DispatcherManagement',
            params={'inputs': inputs},
            response_cls=InsertDatasetVersionResponse,
        )

    @classmethod
    def queryDispatcherRequests(cls, inputs: List[QueryDispatcherRequestsArgs]) -> QueryDispatcherRequestsResponse:
        """
        Search for any DispatcherRequests matching the associated values within Teamcenter.
        
        Use cases:
        Instead of using saved queries to find DispatcherRequests in the system, you would use this method which is
        specific to finding DispatcherRequests.
        """
        return cls.execute_soa_method(
            method_name='queryDispatcherRequests',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DispatcherManagement',
            params={'inputs': inputs},
            response_cls=QueryDispatcherRequestsResponse,
        )

    @classmethod
    def updateDispatcherRequests(cls, inputs: List[UpdateDispatcherRequestArgs]) -> UpdateDispatcherRequestResponse:
        """
        Update DispatcherRequest objects within the Teamcenter database.
        This operation allows updating of the following DispatcherRequest properties:
        - State
        - KeyValue Args
        - Data Files
        
        
        
        Use cases:
        When the need arises to update a DispatcherRequest object, this method provides that ability to update a few
        select properties of the DispatcherRequest.
        """
        return cls.execute_soa_method(
            method_name='updateDispatcherRequests',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DispatcherManagement',
            params={'inputs': inputs},
            response_cls=UpdateDispatcherRequestResponse,
        )

    @classmethod
    def createDatasetOfVersion(cls, inputs: List[CreateDatasetOfVersionArgs]) -> CreateDatasetOfVersionResponse:
        """
        Creates a new Dataset  with a particular version. This is similar to the Core DataManagement createDataset
        operation with the exception that this operation takes a specific version and creates the Dataset at that
        version even if it is the first instance of that Dataset.
        
        Use cases:
        If a Dataset does not already exist within Teamcenter and the applications needs to create one but have it
        begin at a certain version, this operation will provide that ability.  This is a key factor when migrating data
        from other systems where the versions need to stay in sync for validation purposes.
        """
        return cls.execute_soa_method(
            method_name='createDatasetOfVersion',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DispatcherManagement',
            params={'inputs': inputs},
            response_cls=CreateDatasetOfVersionResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def multiRelationMultiLevelExpand(cls, input: MultiRelMultiLevelExpandInput) -> MultiRelationMultiLevelExpandResponse:
        """
        Can be used to expand one or more relations over an entire structure. It recursively expands the relations
        again on the secondary objects of the expand results, until no more relations are found. Branches that lead
        into directed cycles are detected and not followed. So the method will not run into endless loops if the
        structure contains directed cycles.
        """
        return cls.execute_soa_method(
            method_name='multiRelationMultiLevelExpand',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=MultiRelationMultiLevelExpandResponse,
        )

    @classmethod
    def reviseObject(cls, info: List[ReviseInfo], deepCopyRequired: bool) -> ReviseResponse2:
        """
        This operation provides ability to revise all the existing ItemRevision objects given in the input and carry
        forward its relations based on the 'deepCopyRequired' flag. When applying deep copy rules, if user overridden
        deep copy information is provided in the input, then old relations are propagated to the new ItemRevision based
        on the input. If no deep copy information is provided in the input, the deep rules in the database will apply.
        If user provides new property values for the master form in the input, these will be copied to the newly
        created ItemRevision's master form.
        """
        return cls.execute_soa_method(
            method_name='reviseObject',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'info': info, 'deepCopyRequired': deepCopyRequired},
            response_cls=ReviseResponse2,
        )

    @classmethod
    def saveAsNewItemObject(cls, info: List[SaveAsNewItemInfo], deepCopyRequired: bool) -> SaveAsNewItemResponse2:
        """
        This operation provides ability to create new Item objects based on existing ItemRevision objects in the input
        info. It optionally carries forward ItemRevision relations based on the 'deepCopyRequired' flag.  When applying
        deep copy rules, if user overridden deep copy information is provided in the input, then old ItemRevision
        relations are propagated to the new ItemRevision based on the input. If no deep copy rule information is
        provided in the input, the deep rules in the database will be applied. If user provides new property values for
        the Item and ItemRevision master forms in the input, then these will be copied to the master forms of the newly
        created Item and ItemRevision.
        
        Use cases:
        Use this operation to create one or more new Item objects from one or more existing ItemRevision.  The new Item
        and first ItemRevision will be based on the input ItemRevision.
        """
        return cls.execute_soa_method(
            method_name='saveAsNewItemObject',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'info': info, 'deepCopyRequired': deepCopyRequired},
            response_cls=SaveAsNewItemResponse2,
        )

    @classmethod
    def setDefaultProjectForProjectMembers(cls, project: BusinessObject, projectMembers: List[BusinessObject]) -> ServiceData:
        """
        This operation will set the given TC_Project object as user's default project in user context by setting it in
        TC_UserContext object. If the given object is invalid then this operation will return an error with code 7007:
        Invalid tag in the standard ServiceData .
        """
        return cls.execute_soa_method(
            method_name='setDefaultProjectForProjectMembers',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'project': project, 'projectMembers': projectMembers},
            response_cls=ServiceData,
        )

    @classmethod
    def createCachedRelations(cls, input: List[Relationship]) -> CreateRelationsResponse:
        """
        Creates the specified relation between the input objects (primary and secondary objects) for each input
        structure. The relation object created using this service will be cached  in the runtime memory.It also
        initializes all the properties on the relation which are not marked as mandatory for its creation. The service
        operation createRelations, on the other hand does not initialize any property on the new relation it created.
        
        Use cases:
        Use Case 1: Create a relation based on the Generic Relationship Management (GRM) rule definition with property
        on the relation set to a value.
        
        Custom Items, namely, Cus03_tem1 and Cus03_Item2 are related to each other through a relation busuness object
        Cus03_Relation. Cus03_Relation has two properties cus03_relationUsage and cus03_relationUsageDescription. The
        property cus03_relationUsage is a mandatory property at the time of creation of the relation while the property
        cus03_relaionUsageDescription is a property whose initial value cannot be null. Following steps are done by the
        'createCachedRelations' operation. 
        
        - Relation business object is created
        - All properties are initialized to the initial value which is specified in the InitialValue property constant
        for the given property.
        
        """
        return cls.execute_soa_method(
            method_name='createCachedRelations',
            library='Internal-Core',
            service_date='2008_06',
            service_name='DataManagement',
            params={'input': input},
            response_cls=CreateRelationsResponse,
        )


class SessionService(TcService):

    @classmethod
    def cancelOperation(cls, id: str) -> bool:
        """
        Cancel a service request. Currently only 'executeSavedQuery' supports the ability to interrupt and cancel the
        operation.
        """
        return cls.execute_soa_method(
            method_name='cancelOperation',
            library='Internal-Core',
            service_date='2008_06',
            service_name='Session',
            params={'id': id},
            response_cls=bool,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Administration._2011_06.OrganizationManagement import GetOrganizationGroupResponse, GetGroupsForRoleInput
from typing import List
from tcsoa.utils import TcService


class OrganizationManagementService(TcService):

    @classmethod
    def getOrganizationGroups(cls, inputs: List[GetGroupsForRoleInput]) -> GetOrganizationGroupResponse:
        """
        This operation retrieves list of organization groups which contain the given roles. An additional filtered
        organization tree structure which only includes those matched groups for each role could be returned as well
        depending on an option from input parameter. The group tree structure should be included if group hierarchy
        information is needed.
        """
        return cls.execute_soa_method(
            method_name='getOrganizationGroups',
            library='Internal-Administration',
            service_date='2011_06',
            service_name='OrganizationManagement',
            params={'inputs': inputs},
            response_cls=GetOrganizationGroupResponse,
        )



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.utils import TcBaseObj
from dataclasses import dataclass


@dataclass
class InvokeResponse(TcBaseObj):
    """
    This represents the response returned by the Invoke service.
    
    :var xmlOut: The formatted Xml String that the Ai invoke service returns. The structure of xml is an unpublished
    handshake between Teamcenter server and NX client.
    :var data: Service data for this operation. Plain objects are added to this data. All the model objects that are
    contained in the xml output are added here for tracking purpose.
    """
    xmlOut: str = ''
    data: ServiceData = None
